@@ -21,136 +21,10 @@
 #include "libraries.h"
 #include "bible.h"
 #include "utilities.h"
-#include "scripture.h"
 #include <fnmatch.h>
-
-
-char *USFM_IDs[NUMBER_OF_BIBLEBOOKS] =
-  { "GEN", "EXO", "LEV", "NUM", "DEU", "JOS", "JDG", "RUT", "1SA", "2SA",
-    "1KI", "2KI", "1CH", "2CH", "EZR", "NEH", "EST", "JOB", "PSA", "PRO",
-    "ECC", "SNG", "ISA", "JER", "LAM", "EZK", "DAN", "HOS", "JOL", "AMO", "OBA",
-    "JON", "MIC", "NAM", "HAB", "ZEP", "HAG", "ZEC", "MAL",
-    "MAT", "MRK", "LUK", "JHN", "ACT", "ROM", "1CO", "2CO", "GAL", "EPH", "PHP",
-    "COL", "1TH", "2TH", "1TI", "2TI", "TIT", "PHM", "HEB", "JAS",
-    "1PE", "2PE", "1JN", "2JN", "3JN", "JUD", "REV"
-};
-char *BibleWorksIDs[NUMBER_OF_BIBLEBOOKS] =
-  { "Gen", "Exo", "Lev", "Num", "Deu", "Jos", "Jdg", "Rut", "1Sa", "2Sa",
-    "1Ki", "2Ki", "1Ch", "2Ch", "Ezr", "Neh", "Est", "Job", "Psa", "Pro",
-    "Ecc", "Sol", "Isa", "Jer", "Lam", "Eze", "Dan", "Hos", "Joe", "Amo", "Oba",
-    "Jon", "Mic", "Nah", "Hab", "Zep", "Hag", "Zec", "Mal",
-    "Mat", "Mar", "Luk", "Joh", "Act", "Rom", "1Co", "2Co", "Gal", "Eph", "Phi",
-    "Col", "1Th", "2Th", "1Ti", "2Ti", "Tit", "Phm", "Heb", "Jam",
-    "1Pe", "2Pe", "1Jo", "2Jo", "3Jo", "Jud", "Rev"
-};
-char *OSIS_IDs[NUMBER_OF_BIBLEBOOKS] =
-  { "Gen", "Exod", "Lev", "Num", "Deut", "Josh", "Judg", "Ruth", "1Sam",
-    "2Sam", "1Kgs", "2Kgs", "1Chr", "2Chr", "Ezra", "Neh", "Esth", "Job",
-    "Ps", "Prov", "Eccl", "Song", "Isa", "Jer", "Lam", "Ezek", "Dan", "Hos",
-    "Joel", "Amos", "Obad", "Jonah", "Mic", "Nah", "Hab", "Zeph",
-    "Hag", "Zech", "Mal",
-    "Matt", "Mark", "Luke", "John", "Acts", "Rom", "1Cor", "2Cor", "Gal", "Eph",
-    "Phil", "Col", "1Thess", "2Thess", "1Tim", "2Tim", "Titus",
-    "Phlm", "Heb", "Jas", "1Pet", "2Pet", "1John", "2John", "3John", "Jude",
-    "Rev"
-};
-char *EnglishBookNames[NUMBER_OF_BIBLEBOOKS] =
-  { "Genesis", "Exodus", "Leviticus", "Numbers", "Deuteronomy", "Joshua",
-    "Judges", "Ruth", "1 Samuel", "2 Samuel", "1 Kings", "2 Kings",
-    "1 Chronicles", "2 Chronicles", "Ezra", "Nehemiah", "Esther", "Job",
-    "Psalms", "Proverbs", "Ecclesiastes", "Song of Solomon", "Isaiah",
-    "Jeremiah", "Lamentations", "Ezekiel", "Daniel", "Hosea", "Joel", "Amos",
-    "Obadiah", "Jonah", "Micah", "Nahum", "Habakkuk", "Zephaniah",
-    "Haggai", "Zechariah", "Malachi",
-    "Matthew", "Mark", "Luke", "John", "Acts", "Romans", "1 Corinthians",
-    "2 Corinthians", "Galatians", "Ephesians", "Philippians", "Colossians",
-    "1 Thessalonians", "2 Thessalonians", "1 Timothy", "2 Timothy", "Titus",
-    "Philemon", "Hebrews", "James", "1 Peter", "2 Peter", "1 John",
-    "2 John", "3 John", "Jude", "Revelation"
-};
-
-
-ustring id_to_biblebook_english (const ustring & id)
-{
-  // Takes id and returns an English bookname.
-  // E.g. GEN returns "Genesis".
-  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++)
-    {
-      // Note: Allow for mixed case IDs for extra robustness.
-      if (upperCase (id) == USFM_IDs[i])
-        return EnglishBookNames[i];
-    }
-  return "Unknown";
-}
-
-
-ustring index_to_paratext_id (unsigned int index)
-{
-  return USFM_IDs[index];
-}
-
-
-unsigned paratext_id_to_index (const ustring & id)
-{
-  // Returns the index of the Paratext ID. So e.g. EXO returns 1, and GEN 0
-  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++)
-    {
-      // Note: Allow for mixed case IDs for extra robustness.
-      if (upperCase (id) == USFM_IDs[i])
-        return i;
-    }
-  return string::npos;
-}
-
-
-unsigned bibleworks_id_to_index (const ustring & id)
-{
-  // Returns the index of the BibleWorks ID. So e.g. Exo returns 1, and Gen 0
-  ustring id2 = upperCase (id);
-  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
-    if (id2 == upperCase (BibleWorksIDs[i]))
-      return i;
-  }
-  return string::npos;
-}
-
-
-ustring index_to_bibleworks_id (unsigned int index)
-{
-  return BibleWorksIDs[index];
-}
-
-
-unsigned osis_id_to_index (const ustring & id)
-{
-  // Returns the index of the OSIS ID. So e.g. Exod returns 1, and Gen 0
-  ustring id2 = upperCase (id);
-  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++)
-  {
-    if (id2 == upperCase (OSIS_IDs[i]))
-      return i;
-  }
-  return string::npos;
-}
-
-
-ustring index_to_osis_id (unsigned int index)
-// Returns the OSIS id.
-{
-  return OSIS_IDs [index];
-}
-
-
-unsigned  english_id_to_index (const ustring & id)
-{
-  // Returns the index of the English name, e.g. "1Corinthians" or "1 Corinthians"
-  ustring id2 = upperCase (id);
-  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
-    if (id2 == upperCase (EnglishBookNames[i]))
-      return i;
-  }
-  return string::npos;
-}
+#include "projectutils.h"
+#include "books.h"
+#include "session.h"
 
 
 void sort_references (vector <ustring>& references)
@@ -180,12 +54,6 @@ Sorts on book first, then chapter, and finally verse.
 }
 
 
-ustring index_to_biblebook (unsigned int index)
-{
-  return EnglishBookNames[index];
-}
-
-
 void decode_reference (const ustring & reference, ustring & book, ustring & chapter, ustring & verse)
 /*
  * Decodes "reference" and provides:
@@ -235,7 +103,7 @@ bool reference_discover_internal (const ustring& oldbook, const ustring& oldchap
   // Filter the response.
   string response = trim (reference);
   // Change a colon to a space.
-  unsigned int position = response.find (":");
+  size_t position = response.find (":");
   while (position != string::npos) {
     response[position] = ' ';
     position = response.find (":");
@@ -321,7 +189,7 @@ bool reference_discover_internal (const ustring& oldbook, const ustring& oldchap
   // Also as the bookname has been removed from the input data, only chapter and/or verse remain to be dealt with.
   if (input.size () >= 2)
   {
-    // Two variabeles left, so that will be chapter and verse.
+    // Two variables left, so that will be chapter and verse.
     newchapter = number_in_string (input[0]);
     newverse = lowerCase (input[1]);
     if (newchapter.empty () || newverse.empty ())
@@ -330,9 +198,9 @@ bool reference_discover_internal (const ustring& oldbook, const ustring& oldchap
     }
     return true;
   }
-  // Here we have only one variabele left.
+  // Here we have only one variable left.
   // This is either chapter (as in "Genesis 1") or verse (as in "1").
-  // Or we have no variabele left, then we take chapter one verse one.
+  // Or we have no variable left, then we take chapter one verse one.
   if (book_was_given)
   {
     // Here deal with the fact that a book was given, but nothing else.
@@ -343,7 +211,7 @@ bool reference_discover_internal (const ustring& oldbook, const ustring& oldchap
       return true;
     }
   }
-  // One variabele left, so we've either book or verse.
+  // One variable left, so we've either book or verse.
   if (book_was_given)
   {
     // Book was given, so it is a chapter.
@@ -403,102 +271,90 @@ appear cleanly.
 
 
 ustring book_find_valid (const ustring & rawbook)
+// This looks whether "rawbook" can be interpreted as a valid book in any way.
 {
-  /*
-     This sees if variabele "rawbook" can be interpreted as a book in any way.
-   */
+  
   unsigned int index;
   // Check on names entered like Genesis or 1 Corinthians, the full English name
   // A bug was discovered so that "Judges" was interpreted as "Jude", because
   // of the three letters "JUD". Solved by checking on full English name first.
-  index = english_id_to_index (rawbook);
-  if (index != string::npos) {
-    return index_to_biblebook (index);
+  // This also is faster, as most books come in with their normal English names.
+  index = books_english_to_id (rawbook);
+  if (index) {
+    return books_id_to_english (index);
   }
   // Recognise the abbreviations used by Paratext.
-  index = paratext_id_to_index (rawbook);
-  if (index != string::npos) {
-    return index_to_biblebook (paratext_id_to_index (rawbook));
+  index = books_paratext_to_id (rawbook);
+  if (index) {
+    return books_id_to_english (index);
   }
   // Try the abbreviations defined by the OSIS project.
-  index = osis_id_to_index (rawbook);
-  if (index != string::npos)
-  {
-    return index_to_biblebook (index);
+  index = books_osis_to_id (rawbook);
+  if (index) {
+    return books_id_to_english (index);
   }
   // Try the abbreviations of BibleWorks.
-  index = bibleworks_id_to_index (rawbook);
-  if (index != string::npos)
-  {
-    return index_to_biblebook (index);
+  index = books_bibleworks_to_id (rawbook);
+  if (index) {
+    return books_id_to_english (index);
   }
   // Abbreviations in Paratext: See if shortening the bookname helps.
-  if (rawbook.length () >= 3)
-    {
-      ustring s = rawbook.substr (0, 3);
-      index = paratext_id_to_index (s);
-      if (index != string::npos)
-        {
-          return index_to_biblebook (index);
-        }
+  if (rawbook.length () >= 3) {
+    ustring abbrev = rawbook.substr (0, 3);
+    index = books_paratext_to_id (abbrev);
+    if (index) {
+      return books_id_to_english (index);
     }
+  }
   // BibleWorks: See if shortening the bookname helps.
-  if (rawbook.length () >= 3)
-    {
-      ustring s = rawbook.substr (0, 3);
-      index = bibleworks_id_to_index (s);
-      if (index != string::npos)
-        {
-          return index_to_biblebook (index);
-        }
+  if (rawbook.length () >= 3) {
+    ustring abbrev = rawbook.substr (0, 3);
+    index = books_bibleworks_to_id (abbrev);
+    if (index) {
+      return books_id_to_english (index);
     }
+  }
   // OSIS. See if shortening the bookname helps.
   // The shortest abbreviation is 2 characters long,
   // and the longest 6. So we've to try them all.
   // Length: 2
-  if (rawbook.length () >= 2)
-    {
-      ustring s = rawbook.substr (0, 2);
-      index = osis_id_to_index (s);
-      if (index != string::npos)
-        {
-          return index_to_biblebook (index);
-        }
+  if (rawbook.length () >= 2) {
+    ustring abbrev = rawbook.substr (0, 2);
+    index = books_osis_to_id (abbrev);
+    if (index) {
+      return books_id_to_english (index);
     }
+  }
   // Length: 3
-  if (rawbook.length () >= 3)
-    {
-      ustring s = rawbook.substr (0, 3);
-      index = osis_id_to_index (s);
-      if (index != string::npos)
-        {
-          return index_to_biblebook (index);
-        }
+  if (rawbook.length () >= 3) {
+    ustring abbrev = rawbook.substr (0, 3);
+    index = books_osis_to_id (abbrev);
+    if (index) {
+      return books_id_to_english (index);
     }
+  }
   // Length: 4
-  if (rawbook.length () >= 4)
-    {
-      ustring s = rawbook.substr (0, 4);
-      index = osis_id_to_index (s);
-      if (index != string::npos)
-        {
-          return index_to_biblebook (index);
-        }
+  if (rawbook.length () >= 4) {
+    ustring abbrev = rawbook.substr (0, 4);
+    index = books_osis_to_id (abbrev);
+    if (index) {
+      return books_id_to_english (index);
     }
+  }
   // Length: 5
   if (rawbook.length () >= 5) {
-    ustring s = rawbook.substr (0, 5);
-    index = osis_id_to_index (s);
-    if (index != string::npos) {
-      return index_to_biblebook (index);
+    ustring abbrev = rawbook.substr (0, 5);
+    index = books_osis_to_id (abbrev);
+    if (index) {
+      return books_id_to_english (index);
     }
   }
   // Length: 6
   if (rawbook.length () >= 6) {
-    ustring s = rawbook.substr (0, 6);
-    index = osis_id_to_index (s);
-    if (index != string::npos) {
-      return index_to_biblebook (index);
+    ustring abbrev = rawbook.substr (0, 6);
+    index = books_osis_to_id (abbrev);
+    if (index) {
+      return books_id_to_english (index);
     }
   }
   // If the book has not yet been found, then it's getting tough.
@@ -508,9 +364,9 @@ ustring book_find_valid (const ustring & rawbook)
     ustring s2 = rawbook;
     if (s == "1" || s == "2" || s == "3")
       s2.insert (1, " ");
-    index = english_id_to_index (s2);
-    if (index != string::npos) {
-      return index_to_biblebook (index);
+    index = books_english_to_id (s2);
+    if (index) {
+      return books_id_to_english (index);
     }
   }
   return "";
@@ -528,7 +384,7 @@ Etc.
 */
 {
   unsigned int i;
-  i = english_id_to_index (book) * 1000000;
+  i = books_english_to_id (book) * 1000000;
   i = i + (convert_to_int (chapter) * 1000);
   vector<int> verses = verses_encode (verse);
   i = i + verses[0];
@@ -536,14 +392,13 @@ Etc.
 }
 
 
-ustring book_chapter_verse_to_reference (int book, int chapter, const ustring& verse)
+ustring book_chapter_verse_to_reference (const ustring& book, int chapter, const ustring& verse)
 /*
-Changes a book number, with a chapter number, and a verse number, 
+Changes a bookname, with a chapter number, and a verse number, 
 to a full references, e.g. "Genesis 1:1a-4".
 */
 {
-  ustring reference;
-  reference = index_to_biblebook (book);
+  ustring reference (book);
   reference.append (" ");
   reference.append (convert_to_string(chapter));
   reference.append (":");
@@ -552,23 +407,7 @@ to a full references, e.g. "Genesis 1:1a-4".
 }
 
 
-ustring book_chapter_verse_to_reference_paratext (int book, int chapter, int verse)
-/*
-Changes a book number, with a chapter number, and a verse number, 
-to a references as used in Paratext, e.g. "GEN 1:1".
-*/
-{
-  ustring reference;
-  reference = index_to_paratext_id (book);
-  reference.append (" ");
-  reference.append (convert_to_string(chapter));
-  reference.append (":");
-  reference.append (convert_to_string(verse));
-  return reference;
-}
-
-
-ustring book_selection_information (Session * session, const ustring& project)
+ustring book_selection_information (const ustring& project)
 /*
 Returns text to be displayed typically beside the "select books" button.
 The text contains a bit of information about the current selection.
@@ -580,6 +419,9 @@ Old Testament, Mark (2 units)
 etc.
 */
 {
+  // Session data
+  Session session (0);
+  
   // Storage for the units.
   vector<ustring> units;
 
@@ -587,89 +429,43 @@ etc.
   ustring message;
 
   // Deal with empty selection.
-  if (session->selected_books.empty()) {
+  set<ustring> selection = session.selected_books();
+  if (selection.empty()) {
     message = "No books";
     return message;
   }
   
   // See whether OT / NT is in the selection.
-  set<ustring> selection (session->selected_books.begin(), session->selected_books.end());
-
   /*
   At first it only gave "Old Testament" and/or "New Testament" when all 39
-  and/or 27 books were there, using the following code:
-  
-  // Is the whole OT in the selection?
-  bool ot_found = false;
-  for (unsigned int i = 0; i < 39; i++) {
-    ustring book = index_to_biblebook (i);
-    ot_found = (selection.find (book) != selection.end());
-    if (!ot_found)
-      break;
-  }
-
-  // If we found the whole OT, deal with that.
-  if (ot_found) {
-    units.push_back ("Old Testament");
-    for (unsigned int i = 0; i < 39; i++) {
-      ustring book = index_to_biblebook (i);
-      set<ustring>::iterator iter;
-      iter = selection.find(book);
-      selection.erase (iter);
-    }
-  }
-
-  // Is the whole NT in the selection?
-  bool nt_found = false;
-  for (unsigned int i = 39; i < 66; i++) {
-    ustring book = index_to_biblebook (i);
-    nt_found = (selection.find (book) != selection.end());
-    if (!nt_found)
-      break;
-  }
-
-  // If we found the whole NT, deal with that.
-  if (nt_found) {
-    units.push_back ("New Testament");
-    for (unsigned int i = 39; i < 66; i++) {
-      ustring book = index_to_biblebook (i);
-      set<ustring>::iterator iter;
-      iter = selection.find(book);
-      selection.erase (iter);
-    }
-  }
-  
+  and/or 27 books were there.
   But it was preferrable that e.g. "Old Testament" would be given, not only when
   the whole OT was there, but also when all OT books that are currently in the 
   project were there. E.g. if the project contains Genesis and Exodus, and both
-  are selected, it should also give "Old Testament". See the following code.
+  are selected, it should also give "Old Testament".
   */
-  vector<ustring> selectable_books;
-  Scripture scripture (project);
-  for (unsigned int i = 0; i < scripture.books.size (); i++) {
-    selectable_books.push_back (scripture.books[i]);
-  }
+  vector<ustring> selectable_books = project_get_books (project);
 
   // Is the OT in the selection? And the NT?
   unsigned int selectable_ot_books = 0;
   unsigned int selectable_nt_books = 0;
   for (unsigned int i = 0; i < selectable_books.size(); i++) {
-    unsigned index = english_id_to_index (selectable_books[i]);
-    if ((index >= 0) && (index < 39))
+    unsigned index = books_english_to_id (selectable_books[i]);
+    if ((index >= BOOKS_OT_FIRST_ID) && (index <= BOOKS_OT_LAST_ID))
       selectable_ot_books++;
-    if ((index >= 39) && (index < 66))
+    if ((index >= BOOKS_NT_FIRST_ID) && (index <= BOOKS_NT_LAST_ID))
       selectable_nt_books++;
   }
   unsigned int selected_ot_books = 0;
-  for (unsigned int i = 0; i < 39; i++) {
-    ustring book = index_to_biblebook (i);
+  for (unsigned int i = BOOKS_OT_FIRST_ID; i <= BOOKS_OT_LAST_ID; i++) {
+    ustring book = books_id_to_english (i);
     if (selection.find (book) != selection.end())
       selected_ot_books++;
   }
   if ((selected_ot_books > 1) && (selectable_ot_books == selected_ot_books)) {
     units.push_back ("Old Testament");
-    for (unsigned int i = 0; i < 39; i++) {
-      ustring book = index_to_biblebook (i);
+    for (unsigned int i = BOOKS_OT_FIRST_ID; i <= BOOKS_OT_LAST_ID; i++) {
+      ustring book = books_id_to_english (i);
       set<ustring>::iterator iter;
       iter = selection.find(book);
       if (iter != selection.end())
@@ -677,15 +473,15 @@ etc.
     }
   }
   unsigned int selected_nt_books = 0;
-  for (unsigned int i = 39; i < 66; i++) {
-    ustring book = index_to_biblebook (i);
+  for (unsigned int i = BOOKS_NT_FIRST_ID; i <= BOOKS_NT_LAST_ID; i++) {
+    ustring book = books_id_to_english (i);
     if (selection.find (book) != selection.end())
       selected_nt_books++;
   }
   if ((selected_nt_books > 1) && (selectable_nt_books == selected_nt_books)) {
     units.push_back ("New Testament");
-    for (unsigned int i = 39; i < 66; i++) {
-      ustring book = index_to_biblebook (i);
+    for (unsigned int i = BOOKS_NT_FIRST_ID; i <= BOOKS_NT_LAST_ID; i++) {
+      ustring book = books_id_to_english (i);
       set<ustring>::iterator iter;
       iter = selection.find(book);
       if (iter != selection.end())
@@ -698,8 +494,8 @@ etc.
     units.push_back (convert_to_string (int (selection.size())) + " books");
   }
   else {
-    for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
-      ustring book = index_to_biblebook (i);
+    for (unsigned int i = BOOKS_OT_FIRST_ID; i <= BOOKS_NT_LAST_ID; i++) {
+      ustring book = books_id_to_english (i);
       if (selection.find (book) != selection.end()) {
         units.push_back (book);
       }
@@ -817,3 +613,66 @@ So verse 0 becomes then "0, 1", and verse 1 will be "2, 3". Verse 1a will be
   // Return result.
   return expanded_verses;
 }
+
+
+bool chapter_span_discover (const ustring& reference, ustring& chapter1, ustring& verse1, ustring& chapter2, ustring& verse2)
+// Discover whether the reference spans the chapter boundary.
+// E.g. Luke 1:3-2.5
+{
+  // Work on a copy of the reference.
+  ustring ref_in (reference);
+  
+  // Change colons to dots, and parse it into the separate words.
+  replace_text (ref_in, ".", " . ");
+  replace_text (ref_in, ":", " . ");
+  replace_text (ref_in, "-", " - ");
+  Parse parse (ref_in, false);
+
+  // Is the chapter spanning signature (two dots, one hyphen) in it?
+  int hyphencount = 0;
+  int dotcount = 0;
+  for (unsigned int i = 0; i < parse.words.size(); i++) {
+    if (parse.words[i] == "-") hyphencount++;
+    if (parse.words[i] == ".") dotcount++;
+  }
+  if (hyphencount == 0) return false;
+  if (dotcount < 2) return false;
+
+  // There might be chapter spanning at this point.
+  // Are the dots and hyphens in the right order? Should be: dot, hyphen, dot.
+  int firstdotlocation = -1;
+  int seconddotlocation = -1;
+  int hyphenlocation = -1;
+  for (unsigned int i = 0; i < parse.words.size(); i++) {
+    if (parse.words[i] == ".") {
+      if (firstdotlocation == -1) {
+        firstdotlocation = i;
+      } else {
+        seconddotlocation = i;
+      }
+    }
+    if (parse.words[i] == "-") {
+      hyphenlocation = i;
+    }
+  }
+  if (firstdotlocation > hyphenlocation) return false;
+  if (hyphenlocation > seconddotlocation) return false;
+  
+  // We'll take it that there is chapter spanning at this point.
+  // Let's extract the chapter / verse boundaries.
+  if (firstdotlocation > 0) 
+    chapter1 = parse.words[firstdotlocation - 1];
+  verse1 = parse.words[hyphenlocation - 1];
+  chapter2 = parse.words[seconddotlocation - 1];
+  if ((unsigned int) (seconddotlocation + 1) < parse.words.size()) 
+    verse2 = parse.words[seconddotlocation + 1];
+
+  // Do we have all information?
+  if (chapter1.empty()) return false;
+  if (verse1.empty()) return false;
+  if (chapter2.empty()) return false;
+  if (verse2.empty()) return false;
+  
+  // Right then, it spans the chapter boundary.
+  return true;
+}
