@@ -25,14 +25,12 @@
 #include <config.h>
 #include "constants.h"
 #include "compareutils.h"
-#include "scripture.h"
-#include "book.h"
 #include "usfmtools.h"
 #include "usfm.h"
 #include "printproject.h"
 #include "printreferences.h"
 #include "projectutils.h"
-#include "bookmetrics.h"
+#include "generalconfig.h"
 
 
 enum Markup { StrikeThrough, Bold };
@@ -53,103 +51,90 @@ void compare_word_by_word_internal (
 
 
 
-void compare_with (Configuration * configuration, Progress * progress)
+void compare_with ()
 {
+  // Configuration
+  GeneralConfiguration genconfig (0);
   // Progress system.
-  progress->start();
-  // GUI info.
-  progress->set_text ("Preparing projects");
-  progress->set_fraction (0.1);
+  ProgressWindow progresswindow ("Comparing", true);
+  progresswindow.set_text ("Preparing projects");
+  progresswindow.set_fraction (0.1);
   // Copy current project to a new project.
   ustring newproject;
-  newproject = configuration->project + " compared with " + configuration->project_to_compare_with;
-  projectutils_copy_project (configuration->project, newproject);
-  if (progress->cancel) {
-    progress->finish();
+  newproject = genconfig.project() + " compared with " + genconfig.project_to_compare_with();
+  project_copy (genconfig.project(), newproject);
+  if (progresswindow.cancel) {
     return;
   }
   // Do the actual comparison.
-  compare_projects (configuration->project, configuration->project_to_compare_with, newproject, progress);
-  if (progress->cancel) {
-    progress->finish();
+  compare_projects (genconfig.project(), genconfig.project_to_compare_with(), newproject, progresswindow);
+  if (progresswindow.cancel) {
     return;
   }
   // See what to print.
-  progress->set_text ("Printing differences");
-  string project = configuration->project;
-  configuration->project = newproject;
-  if (configuration->print_changes_only) {
+  progresswindow.set_text ("Printing differences");
+  string project = genconfig.project();
+  genconfig.project_set (newproject);
+  if (genconfig.print_changes_only()) {
     // Print changes only.
     vector<ustring> references;
     compare_get_changes (newproject, references);
-    Session session (0);
-    view_references_pdf (configuration, references, &session, progress);
+    view_references_pdf (references, progresswindow);
   } else {
     // Print whole modified project.
-    view_project_pdf (configuration, progress);
+    view_project_pdf (progresswindow);
   }
-  configuration->project = project;
-  if (progress->cancel) {
-    progress->finish();
+  genconfig.project_set (project);
+  if (progresswindow.cancel) {
     return;
   }
   // Delete the modified project.
-  projectutils_delete_project (newproject);
-  progress->finish();
+  project_delete (newproject);
 }
 
 
-void compare_projects (const ustring& originalproject, const ustring& secondproject, 
-                       const ustring& outputproject, Progress * progress)
+void compare_projects (const ustring& originalproject, const ustring& secondproject,
+                       const ustring& outputproject, ProgressWindow& progresswindow)
 /*
 Compares originalproject with secondproject, 
 and puts the differences in outputproject.
 */
 {
   // Progress information.
-  progress->set_text ("Starting comparison");
+  progresswindow.set_text ("Starting comparison");
   // Open the Scriptures.
-  Scripture originalscripture (originalproject);
-  Scripture secondscripture (secondproject);
-  Scripture outputscripture (outputproject);
+  vector<ustring> originalscripture_books = project_get_books (originalproject);
+  vector<ustring> secondscripture_books = project_get_books (secondproject);
+  vector<ustring> outputscripture = project_get_books (outputproject);
   // Progress information.
-  progress->set_iterate (0.1, 0.5, originalscripture.books.size());
+  progresswindow.set_iterate (0.1, 0.5, originalscripture_books.size());
   // Go through the original scripture.
-  for (unsigned int ib = 0; ib < originalscripture.books.size(); ib++) {
+  for (unsigned int ib = 0; ib < originalscripture_books.size(); ib++) {
     // Progress.
-    progress->iterate ();
-    // If this book does not exist in the second project continue to the next book,
-    // without comparing.
-    int secondproject_bookindex;
-    secondproject_bookindex = secondscripture.get_index_of_book (originalscripture.books[ib]);
-    if (secondproject_bookindex < 0)
+    progresswindow.iterate ();
+    // If this book does not exist in the second project,
+    // continue to the next book, without comparing.
+    if (!project_book_exists (secondproject, originalscripture_books[ib]))
       continue;
-    if (progress->cancel) {
-      progress->finish();
+    if (progresswindow.cancel) {
       return;
     }
-    // Open the two books to compare, and the output book and the metrics too.
-    Book originalbook (originalscripture.paths[ib], false, "");
-    BookMetrics originalmetrics (originalscripture.paths[ib]);
-    Book secondbook (secondscripture.paths[secondproject_bookindex], false, "");
-    Book outputbook (outputscripture.paths[ib], false, "");
-    // Go through each chapter.
+    // Go through each chapter of the original book.
     vector<unsigned int> originalchapters;
-    originalchapters = originalmetrics.get_chapters();
+    originalchapters = project_get_chapters (originalproject, originalscripture_books[ib]);
     for (unsigned int ic = 1; ic < originalchapters.size(); ic++) {
       // Progress info.
-      ustring info = originalscripture.books[ib] + " " + convert_to_string (originalchapters[ic]);
-      progress->set_text (info);
-      if (progress->cancel) {
-        progress->finish();
+      ustring info = originalscripture_books[ib] + " " + convert_to_string (originalchapters[ic]);
+      progresswindow.set_text (info);
+      if (progresswindow.cancel) {
         return;
       }
       // Get chapter of the original.
       vector <ustring> originallines;
-      originalbook.get_chapter (originalchapters[ic], originallines);
+      originallines = project_retrieve_chapter (originalproject, originalscripture_books[ib], originalchapters[ic]);
       // Get chapter of the second.
       vector <ustring> secondlines;
-      secondbook.get_chapter (originalchapters[ic], secondlines);
+      secondlines = project_retrieve_chapter (secondproject, originalscripture_books[ib], originalchapters[ic]);
       // If both chapters are the same, continue to the next one.
       ustring originalbook_chapter_contents;
       for (unsigned int i = 0; i < originallines.size(); i++)
@@ -161,10 +146,10 @@ and puts the differences in outputproject.
         continue;
       // At this stage the chapters are different. Compare them more thoroughly.
       vector <ustring> outputlines;
-      outputbook.get_chapter (originalchapters[ic], outputlines);
+      outputlines = project_retrieve_chapter (outputproject, originalscripture_books[ib], originalchapters[ic]);
       compare_chapter (originallines, secondlines, outputlines);
-      outputbook.set_chapter (outputlines);
-      outputbook.save();
+      CategorizeChapterVerse ccv (outputlines);
+      project_store_chapter (outputproject, originalscripture_books[ib], ccv);
     }      
   }
 }
@@ -233,7 +218,7 @@ process also, to find out which algorithm detects the lowest number of changes.
 {
   // Variabeles and initialization of them.
   number_of_changes = 0;
-  // Save the second chapter to a temporal variabele.
+  // Save the second chapter to a temporal variable.
   vector<ustring> usfm2 (secondusfms.begin(), secondusfms.end());
   vector<ustring> text2 (secondtext.begin(), secondtext.end());
   // Go through all the original lines.
@@ -449,17 +434,15 @@ void compare_get_changes (const ustring& project, vector <ustring>& changed_refe
 // This produces all changed references in a project.
 {
   // Go through the whole scripture.
-  Scripture scripture (project);
-  for (unsigned int i = 0; i < scripture.books.size (); i++) {
-    scripture.books[i];
-    Book book (scripture.paths[i], false, "");
-    BookMetrics bookmetrics (scripture.paths[i]);
-    for (unsigned int chapter = 1; chapter < bookmetrics.get_chapters().size(); chapter++) {
+  vector <ustring> scripture_books = project_get_books (project);
+  for (unsigned int i = 0; i < scripture_books.size (); i++) {
+    vector <unsigned int> chapters = project_get_chapters (project, scripture_books[i]);
+    for (unsigned int chapter = 1; chapter < chapters.size(); chapter++) {
       vector <ustring> lines;
-      book.get_chapter (chapter, lines);
-      string book_chapter = scripture.books[i] + " " + convert_to_string (chapter) + ":";
-      string currentverse = "1";
-      string last_reference_stored;
+      lines = project_retrieve_chapter (project, scripture_books[i], chapter);
+      ustring book_chapter = scripture_books[i] + " " + convert_to_string (chapter) + ":";
+      ustring currentverse = "1";
+      ustring last_reference_stored;
       for (unsigned int i2 = 0; i2 < lines.size (); i2++) {
         ustring line (lines[i2]);
         // Extract the verse number.
