@@ -22,12 +22,12 @@
 
 
 #include "libraries.h"
-#include "progress.h"
+#include "progresswindow.h"
 
 
-void compare_with (Configuration * configuration, Progress * progress);
+void compare_with ();
 void compare_projects (const ustring& originalproject, const ustring& secondproject, 
-                       const ustring& outputproject, Progress * progress);
+                       const ustring& outputproject, ProgressWindow& progresswindow);
 void compare_chapter (vector<ustring>& originalchapter, vector<ustring>& secondchapter, vector<ustring>& outputchapter);
 void compare_get_changes (const ustring& project, vector <ustring>& changed_references);
 
