@@ -22,7 +22,6 @@
 
 
 #include <gtk/gtk.h>
-#include "configuration.h"
 #include "ustring.h"
 
 
@@ -30,8 +29,8 @@ class EntryDialog
 {
 public:
   EntryDialog (const ustring& title, const ustring& info, const ustring& value);
-  virtual ~EntryDialog ();
-  virtual int run ();
+  ~EntryDialog ();
+  int run ();
   ustring entered_value;
 protected:
   GtkWidget *entrydialog;
@@ -44,9 +43,9 @@ protected:
   GtkWidget *okbutton;
 private:
   static void on_okbutton_clicked (GtkButton *button, gpointer user_data);
-  virtual void on_okbutton ();
+  void on_okbutton ();
   static void on_entry_changed (GtkEditable *editable, gpointer user_data);
-  virtual void on_entry ();
+  void on_entry ();
 };
 
 
