@@ -30,18 +30,20 @@
 #include "directories.h"
 #include "gwrappers.h"
 #include "shell.h"
+#include "generalconfig.h"
+#include "gui.h"
 
 
-FormatterDialog::FormatterDialog (Configuration* configuration)
+FormatterDialog::FormatterDialog (int dummy)
 {
   // Save and initialize variables.
-  myconfiguration = configuration;
-  java_binary_path = configuration->java_binary;
+  GeneralConfiguration genconfig (0);
+  java_binary_path = genconfig.java_binary();
   xep_package_path = g_get_home_dir ();
   xep_package_path.append ("/");
   license_path = g_get_home_dir ();
   license_path.append ("/");
-  xep_home = configuration->xep_home;
+  xep_home = genconfig.xep_home();
   
   // Build GUI.
   formatterdialog = gtk_dialog_new ();
@@ -57,7 +59,7 @@ FormatterDialog::FormatterDialog (Configuration* configuration)
   gtk_widget_show (vbox1);
   gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);
 
-  table1 = gtk_table_new (8, 3, FALSE);
+  table1 = gtk_table_new (8, 4, FALSE);
   gtk_widget_show (table1);
   gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);
   gtk_table_set_row_spacings (GTK_TABLE (table1), 4);
@@ -69,16 +71,23 @@ FormatterDialog::FormatterDialog (Configuration* configuration)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 
-  label24 = gtk_label_new ("1. Open the Java binary");
+  label_java_gui = gtk_label_new ("");
+  gtk_widget_show (label_java_gui);
+  gtk_table_attach (GTK_TABLE (table1), label_java_gui, 1, 2, 0, 1,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_java_gui), 0, 0.5);
+
+  label24 = gtk_label_new ("Open the Java binary");
   gtk_widget_show (label24);
-  gtk_table_attach (GTK_TABLE (table1), label24, 1, 2, 0, 1,
+  gtk_table_attach (GTK_TABLE (table1), label24, 2, 3, 0, 1,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label24), 0, 0.5);
 
   button_java = gtk_button_new ();
   gtk_widget_show (button_java);
-  gtk_table_attach (GTK_TABLE (table1), button_java, 2, 3, 0, 1,
+  gtk_table_attach (GTK_TABLE (table1), button_java, 3, 4, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
 
@@ -111,16 +120,23 @@ FormatterDialog::FormatterDialog (Configuration* configuration)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 
-  label31 = gtk_label_new ("4. Install or upgrade XEP");
+  label_install_gui = gtk_label_new ("");
+  gtk_widget_show (label_install_gui);
+  gtk_table_attach (GTK_TABLE (table1), label_install_gui, 1, 2, 6, 7,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_install_gui), 0, 0.5);
+
+  label31 = gtk_label_new ("Install or upgrade XEP");
   gtk_widget_show (label31);
-  gtk_table_attach (GTK_TABLE (table1), label31, 1, 2, 6, 7,
+  gtk_table_attach (GTK_TABLE (table1), label31, 2, 3, 6, 7,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label31), 0, 0.5);
 
   button_install = gtk_button_new ();
   gtk_widget_show (button_install);
-  gtk_table_attach (GTK_TABLE (table1), button_install, 2, 3, 6, 7,
+  gtk_table_attach (GTK_TABLE (table1), button_install, 3, 4, 6, 7,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
 
@@ -153,16 +169,23 @@ FormatterDialog::FormatterDialog (Configuration* configuration)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 
-  label29 = gtk_label_new ("3. Open the XEP license");
+  label_license_gui = gtk_label_new ("ok");
+  gtk_widget_show (label_license_gui);
+  gtk_table_attach (GTK_TABLE (table1), label_license_gui, 1, 2, 4, 5,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_license_gui), 0, 0.5);
+
+  label29 = gtk_label_new ("Open the XEP license");
   gtk_widget_show (label29);
-  gtk_table_attach (GTK_TABLE (table1), label29, 1, 2, 4, 5,
+  gtk_table_attach (GTK_TABLE (table1), label29, 2, 3, 4, 5,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label29), 0, 0.5);
 
   button_license = gtk_button_new ();
   gtk_widget_show (button_license);
-  gtk_table_attach (GTK_TABLE (table1), button_license, 2, 3, 4, 5,
+  gtk_table_attach (GTK_TABLE (table1), button_license, 3, 4, 4, 5,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
 
@@ -195,16 +218,23 @@ FormatterDialog::FormatterDialog (Configuration* configuration)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 
-  label27 = gtk_label_new ("2. Open the XEP installation package");
+  label_xep_gui = gtk_label_new ("ok");
+  gtk_widget_show (label_xep_gui);
+  gtk_table_attach (GTK_TABLE (table1), label_xep_gui, 1, 2, 2, 3,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (0), 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label_xep_gui), 0, 0.5);
+
+  label27 = gtk_label_new ("Open the XEP installation package");
   gtk_widget_show (label27);
-  gtk_table_attach (GTK_TABLE (table1), label27, 1, 2, 2, 3,
+  gtk_table_attach (GTK_TABLE (table1), label27, 2, 3, 2, 3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label27), 0, 0.5);
 
   button_xep = gtk_button_new ();
   gtk_widget_show (button_xep);
-  gtk_table_attach (GTK_TABLE (table1), button_xep, 2, 3, 2, 3,
+  gtk_table_attach (GTK_TABLE (table1), button_xep, 3, 4, 2, 3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
 
@@ -288,10 +318,11 @@ void FormatterDialog::on_okbutton_clicked (GtkButton *button, gpointer user_data
 
 void FormatterDialog::on_okbutton ()
 {
+  GeneralConfiguration genconfig (0);
   // Store Java just to remember for next time.
-  myconfiguration->java_binary = java_binary_path;
+  genconfig.java_binary_set (java_binary_path);
   // Store xep home for printing.
-  myconfiguration->xep_home = xep_home;
+  genconfig.xep_home_set (xep_home);
 }
 
 
@@ -432,7 +463,7 @@ void FormatterDialog::set_gui ()
   }
 
   // Set in gui whether java was ok.
-  image_ok (image_java, java_ok);
+  gui_okay (image_java, label_java_gui, java_ok);
   java_information = trim (java_information);
   if (java_ok)
     java_information.insert (0, "Binary: " + java_binary_path + "\n");
@@ -444,7 +475,7 @@ void FormatterDialog::set_gui ()
   bool xep_ok = find_installation_package ();
   
   // Set in gui whether the package is fine.
-  image_ok (image_xep, xep_ok);
+  gui_okay (image_xep, label_xep_gui, xep_ok);
   ustring xep_information (xep_package_path);
   if (xep_ok) 
     xep_information.insert (0, "Package is ");
@@ -456,7 +487,7 @@ void FormatterDialog::set_gui ()
   bool license_ok = find_license ();
   
   // Set in gui whether the license is fine.
-  image_ok (image_license, license_ok);
+  gui_okay (image_license, label_license_gui, license_ok);
   ustring license_information (license_path);
   if (license_ok) 
     license_information.insert (0, "License is ");
@@ -482,16 +513,7 @@ void FormatterDialog::set_gui ()
   else
     xep.insert (0, "XEP installation not found in ");
   gtk_label_set_text (GTK_LABEL (label_install), xep.c_str());
-  image_ok (image_install, xep_installed);
-}
-
-
-void FormatterDialog::image_ok (GtkWidget * image, bool ok)
-{
-  if (ok)
-    gtk_image_set_from_stock (GTK_IMAGE (image), "gtk-yes", GTK_ICON_SIZE_BUTTON);
-  else
-    gtk_image_set_from_stock (GTK_IMAGE (image), "gtk-no", GTK_ICON_SIZE_BUTTON);
+  gui_okay (image_install, label_install_gui, xep_installed);
 }
 
 
