@@ -32,6 +32,7 @@
 #include "gtkwrappers.h"
 #include "gwrappers.h"
 #include "shell.h"
+#include "books.h"
 
 
 #define TEMP_FILE "bibledit.import.notes"
@@ -605,8 +606,8 @@ When we encounter an element that ends data, this handler deals with that.
             reference_content.append (convert_to_string (int (book_chapter + verses[i2])));
           }
           // Store the reference in OSIS format too.
-          int index = english_id_to_index (book);
-          ustring osis_book = index_to_osis_id (index);
+          unsigned int index = books_english_to_id (book);
+          ustring osis_book = books_id_to_osis (index);
           ustring osis_ref = osis_book + "." + chapter + "." + verse;
           if (!osis_reference.empty())
             osis_reference.append (" ");
