@@ -30,17 +30,16 @@
 #include "usfmtools.h"
 #include "stylesheetutils.h"
 #include "shell.h"
+#include "generalconfig.h"
+#include "session.h"
 
 
 enum NumberingType {ntAutomatic, ntNone, ntCharacter};
 
 
-InsertNoteDialog::InsertNoteDialog (Configuration* configuration, Session * session, 
-                                    NoteType dialogtype,
-                                    GtkTextBuffer * textbuffer)
+InsertNoteDialog::InsertNoteDialog (NoteType dialogtype, GtkTextBuffer * textbuffer)
 {
-  myconfiguration = configuration;
-  mysession = session;
+  GeneralConfiguration genconfig (0);
   mydialogtype = dialogtype;
   mytextbuffer = textbuffer;
   allow_entry_activate = true;
@@ -242,7 +241,7 @@ InsertNoteDialog::InsertNoteDialog (Configuration* configuration, Session * sess
 
   // Load available content markers.
   vector<Style> styles;
-  stylesheet_get_styles (configuration->stylesheet, styles);
+  stylesheet_get_styles (genconfig.stylesheet(), styles);
   for (unsigned int i = 0; i < styles.size(); i++) {
     switch (dialogtype) {
       case indtFootnote:
@@ -280,13 +279,13 @@ InsertNoteDialog::InsertNoteDialog (Configuration* configuration, Session * sess
   ustring template_name;
   switch (dialogtype) {
     case indtFootnote:
-      template_name = configuration->insert_footnote_template;
+      template_name = genconfig.insert_footnote_template();
       break;
     case indtEndnote:
-      template_name = configuration->insert_endnote_template;
+      template_name = genconfig.insert_endnote_template();
       break;
     case indtCrossreference:
-      template_name = configuration->insert_xref_template;
+      template_name = genconfig.insert_xref_template();
       break;
   }
   templates_load (template_name);
@@ -457,15 +456,17 @@ ustring InsertNoteDialog::reference_get ()
 {
   // Variable for result.
   ustring result;
+  // Configuration
+  GeneralConfiguration genconfig (0);
   // If the separator widget is sensitive, get the chapter and verse.
   if (GTK_WIDGET_SENSITIVE (entry_reference_separator)) {
-    result = myconfiguration->chapter_opened 
+    result = genconfig.chapter() 
            + gtk_entry_get_text (GTK_ENTRY (entry_reference_separator))
-           + myconfiguration->verse_opened;
+           + genconfig.verse();
   } else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_chapter))) {
-    result = myconfiguration->chapter_opened;
+    result = genconfig.chapter();
   } else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_verse))) {
-    result = myconfiguration->verse_opened;
+    result = genconfig.verse();
   }
   // Add the suffix.
   result.append (gtk_entry_get_text (GTK_ENTRY (entry_reference_suffix)));
@@ -825,6 +826,7 @@ void InsertNoteDialog::on_okbutton1_clicked (GtkButton *button, gpointer user_da
 void InsertNoteDialog::on_okbutton ()
 {
   // Save the current gui.
+  GeneralConfiguration genconfig (0);
   NoteTemplate notetemplate (gw_build_filename (directories_get_configuration (), template_filename_get (combobox_get_active_string (combobox_templates))), mydialogtype, true);
   notetemplate.numbering = numbering_get ();
   notetemplate.anchor = gtk_entry_get_text (GTK_ENTRY (entry_numbering));
@@ -844,17 +846,17 @@ void InsertNoteDialog::on_okbutton ()
   switch (mydialogtype) {
     case indtFootnote:
     {
-      myconfiguration->insert_footnote_template = combobox_get_active_string (combobox_templates);
+      genconfig.insert_footnote_template_set (combobox_get_active_string (combobox_templates));
       break;
     }
     case indtEndnote:
     {
-      myconfiguration->insert_endnote_template = combobox_get_active_string (combobox_templates);
+      genconfig.insert_endnote_template_set (combobox_get_active_string (combobox_templates));
       break;
     }
     case indtCrossreference:
     {
-      myconfiguration->insert_xref_template = combobox_get_active_string (combobox_templates);
+      genconfig.insert_xref_template_set (combobox_get_active_string (combobox_templates));
       break;
     }
   }
@@ -926,3 +928,7 @@ void InsertNoteDialog::on_okbutton ()
   // Insert the note in the editor.
   gtk_text_buffer_insert_at_cursor (mytextbuffer, note.c_str(), -1);
 }
+
+
+// Todo give more space when using \ft.
+// Todo Note templates. Instead of “new' do “save as”, and keep the settings as they are.
