@@ -22,22 +22,20 @@
 #include <glib.h>
 #include "dialogprintorderportion.h"
 #include "utilities.h"
-#include "scripture.h"
 #include "bible.h"
-#include "bookmetrics.h"
 #include "dialogselectchapters.h"
-#include "project.h"
+#include "projectutils.h"
+#include "books.h"
+#include "scripture_reordered.h"
+#include "projectconfig.h"
 
 
 enum { COLUMN_BOOK, COLUMN_INCLUDE, COLUMN_PORTION, NUM_COLUMNS };
 
 
-PrintOrderPortionDialog::PrintOrderPortionDialog (Scripture * scripture, Configuration * configuration)
+PrintOrderPortionDialog::PrintOrderPortionDialog (int dummy):
+  genconfig (0)
 {
-  // Store data.
-  myscripture = scripture;
-  myconfiguration = configuration;
-
   printorderportiondialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (printorderportiondialog), "Book order and portion");
   gtk_window_set_position (GTK_WINDOW (printorderportiondialog), GTK_WIN_POS_CENTER_ON_PARENT);
@@ -261,13 +259,12 @@ PrintOrderPortionDialog::PrintOrderPortionDialog (Scripture * scripture, Configu
   gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
 
   // Load content.
-  Project project (configuration->project);
-  project.reordering_read ();
+  ScriptureReordered scripture_reordered (genconfig.project());
   GtkTreeIter iter;
-  for (unsigned int i = 0; i < project.reordered_books.size(); i++) {
+  for (unsigned int i = 0; i < scripture_reordered.reordered_books.size(); i++) {
     gtk_list_store_append (store, &iter);
-    bool include = project.reordered_includes[i];  
-    gtk_list_store_set (store, &iter, COLUMN_BOOK, project.reordered_books[i].c_str (), COLUMN_INCLUDE, include, COLUMN_PORTION, project.reordered_portions[i].c_str(), -1);
+    bool include = scripture_reordered.reordered_includes[i];  
+    gtk_list_store_set (store, &iter, COLUMN_BOOK, scripture_reordered.reordered_books[i].c_str (), COLUMN_INCLUDE, include, COLUMN_PORTION, scripture_reordered.reordered_portions[i].c_str(), -1);
   }
 
   // Set rest of gui.
@@ -295,8 +292,10 @@ void PrintOrderPortionDialog::on_okbutton1_clicked (GtkButton *button, gpointer
 
 void PrintOrderPortionDialog::on_okbutton ()
 {
-  // Store data in project.
-  Project project (myconfiguration);
+  // Containers.
+  vector<ustring> reordered_books;
+  vector<bool> reordered_includes;
+  vector<ustring> reordered_portions;
   // Get the model
   GtkTreeModel * model;
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview1));
@@ -315,9 +314,9 @@ void PrintOrderPortionDialog::on_okbutton ()
     gboolean include_data;
     gchar *str_portion;
     gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COLUMN_BOOK, &str_book, COLUMN_INCLUDE, &include_data, COLUMN_PORTION, &str_portion, -1);
-    project.reordered_books.push_back (str_book);
-    project.reordered_includes.push_back (include_data);
-    project.reordered_portions.push_back (str_portion);
+    reordered_books.push_back (str_book);
+    reordered_includes.push_back (include_data);
+    reordered_portions.push_back (str_portion);
     // Free some data.
     gtk_tree_path_free (path);
     g_free (str_book);
@@ -327,7 +326,10 @@ void PrintOrderPortionDialog::on_okbutton ()
     index++;
   }
   // Save reordered books.
-  project.reordering_write ();
+  ProjectConfiguration projectconfig ("");
+  projectconfig.reordered_books_set (reordered_books);
+  projectconfig.reordered_includes_set (reordered_includes);
+  projectconfig.reordered_portions_set (reordered_portions);
 }
 
 
@@ -340,13 +342,8 @@ void PrintOrderPortionDialog::on_button_ot_clicked (GtkButton *button, gpointer
 void PrintOrderPortionDialog::on_button_ot ()
 {
   set <ustring> books;
-  unsigned int start = english_id_to_index ("Genesis");
-  unsigned int end = english_id_to_index ("Malachi");
-  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
-    if ((i >= start) && (i <= end)) {
-      books.insert (id_to_biblebook_english (index_to_paratext_id (i)));      
-    }
-  }
+  for (unsigned int i = BOOKS_OT_FIRST_ID; i <= BOOKS_OT_LAST_ID; i++)
+    books.insert (books_id_to_english (i));
   books_include (books);
 }
 
@@ -360,13 +357,8 @@ void PrintOrderPortionDialog::on_button_nt_clicked (GtkButton *button, gpointer
 void PrintOrderPortionDialog::on_button_nt ()
 {
   set <ustring> books;
-  unsigned int start = english_id_to_index ("Matthew");
-  unsigned int end = english_id_to_index ("Revelation");
-  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
-    if ((i >= start) && (i <= end)) {
-      books.insert (id_to_biblebook_english (index_to_paratext_id (i)));      
-    }
-  }
+  for (unsigned int i = BOOKS_NT_FIRST_ID; i <= BOOKS_NT_LAST_ID; i++)
+    books.insert (books_id_to_english (i));
   books_include (books);
 }
 
@@ -379,7 +371,8 @@ void PrintOrderPortionDialog::on_button_all_clicked (GtkButton *button, gpointer
 
 void PrintOrderPortionDialog::on_button_all ()
 {
-  set <ustring> books (myscripture->books.begin(), myscripture->books.end());
+  vector<ustring> scripture_books = project_get_books (genconfig.project());
+  set <ustring> books (scripture_books.begin(), scripture_books.end());
   books_include (books);
 }
 
@@ -393,7 +386,7 @@ void PrintOrderPortionDialog::on_button_current_clicked (GtkButton *button, gpoi
 void PrintOrderPortionDialog::on_button_current ()
 {
   set <ustring> books;
-  books.insert (myconfiguration->book_opened);
+  books.insert (genconfig.book());
   books_include (books);
 }
 
@@ -413,9 +406,9 @@ void PrintOrderPortionDialog::on_button_standard_order ()
   GtkTreeSelection * selection;
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview1));
   // Go through all the Bible books.
-  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
+  for (unsigned int i = BOOKS_OT_FIRST_ID; i <= BOOKS_NT_LAST_ID; i++) {
     // Get English bookname, e.g. Matthew.
-    ustring book = id_to_biblebook_english (index_to_paratext_id (i));
+    ustring book = books_id_to_english (i);
     // Some variables needed to iterate through the model.
     GtkTreeIter iter;
     gboolean valid;
@@ -478,9 +471,7 @@ void PrintOrderPortionDialog::on_button_portion ()
   if (!selection_get (book, include, portion))
     return;
   // Run the dialog to select a portion.
-  int index = myscripture->get_index_of_book (book);
-  BookMetrics bookmetrics (myscripture->paths[index]);
-  SelectChaptersDialog dialog (&bookmetrics, portion);
+  SelectChaptersDialog dialog (genconfig.project(), book, portion);
   if (dialog.run() == GTK_RESPONSE_OK) {
     // Ok, we've new portion: store it.
     // Some variables needed.
