@@ -24,15 +24,15 @@
 
 #include <gtk/gtk.h>
 #include "libraries.h"
-#include "configuration.h"
+#include "generalconfig.h"
 
 
 class ScreenLayoutDialog
 {
 public:
-  ScreenLayoutDialog (Configuration * configuration);
-  virtual ~ScreenLayoutDialog ();
-  virtual int run();
+  ScreenLayoutDialog (int dummy);
+  ~ScreenLayoutDialog ();
+  int run();
 protected:
   GtkWidget *screenlayoutdialog;
   GtkWidget *dialog_vbox1;
@@ -44,8 +44,7 @@ protected:
   GtkWidget *okbutton1;
 private:
   static void on_okbutton1_clicked (GtkButton *button, gpointer user_data);
-  virtual void on_ok ();
-  Configuration * myconfiguration;
+  void on_ok ();
 };
 
 
