@@ -21,13 +21,14 @@
 #include "libraries.h"
 #include "dialogsearchspecial.h"
 #include "dialogselectbooks.h"
-#include "scripture.h"
 #include "completion.h"
 #include "combobox.h"
 #include <gdk/gdkkeysyms.h>
 #include "keyboard.h"
 #include "bible.h"
 #include "dialogarea.h"
+#include "projectutils.h"
+#include "generalconfig.h"
 
 
 #define RESULTS_LOAD   "Load them in the References Area"
@@ -36,10 +37,10 @@
 #define RESULTS_SHARE  "Share them with the ones already in the References Area"
 
 
-SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session * session, BibleTime * bibletime)
+SearchSpecialDialog::SearchSpecialDialog (BibleTime * bibletime) :
+  session (0)
 {
-  myconfiguration = configuration;
-  mysession = session;
+  GeneralConfiguration genconfig (0);
   mybibletime = bibletime;
 
   accel_group = gtk_accel_group_new ();
@@ -104,7 +105,7 @@ SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session
   gtk_box_pack_start (GTK_BOX (hbox1), word_entry, TRUE, TRUE, 0);
   
   // Set search text.
-  gtk_entry_set_text (GTK_ENTRY (word_entry), session->searchword.c_str());
+  gtk_entry_set_text (GTK_ENTRY (word_entry), session.searchword().c_str());
 
   // Activation of dialog.
   allow_activate = true;
@@ -125,15 +126,22 @@ SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session
   gtk_box_pack_start (GTK_BOX (vbox6), checkbutton_casesensitive, FALSE, FALSE, 0);
 
   // Set case sensitive.
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_casesensitive), mysession->search_case_sensitive);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_casesensitive), session.search_case_sensitive());
 
   checkbutton_current_book = gtk_check_button_new_with_mnemonic ("Search in c_urrent book only");
   gtk_widget_show (checkbutton_current_book);
   gtk_box_pack_start (GTK_BOX (vbox6), checkbutton_current_book, FALSE, FALSE, 0);
 
   // Set searching in current book.
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_current_book), mysession->search_current_book);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_current_book), session.search_current_book());
 
+  checkbutton_current_chapter = gtk_check_button_new_with_mnemonic ("Search in current c_hapter only");
+  gtk_widget_show (checkbutton_current_chapter);
+  gtk_box_pack_start (GTK_BOX (vbox6), checkbutton_current_chapter, FALSE, FALSE, 0);
+
+  // Set searching in current chapter.
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_current_chapter), session.search_current_chapter());
+  
   label27 = gtk_label_new ("");
   gtk_widget_show (label27);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_case_current), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_case_current), 0), label27);
@@ -179,21 +187,21 @@ SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session
   gtk_box_pack_start (GTK_BOX (vbox5), checkbutton_glob, FALSE, FALSE, 0);
 
   // Set active or not.
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_glob), mysession->search_globbing);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_glob), session.search_globbing());
 
   checkbutton_start_word = gtk_check_button_new_with_mnemonic ("Match _start of word");
   gtk_widget_show (checkbutton_start_word);
   gtk_box_pack_start (GTK_BOX (vbox5), checkbutton_start_word, FALSE, FALSE, 0);
 
   // Set active or not.
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_start_word), mysession->search_start_word_match);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_start_word), session.search_start_word_match());
 
   checkbutton_end_word = gtk_check_button_new_with_mnemonic ("Match _end of word");
   gtk_widget_show (checkbutton_end_word);
   gtk_box_pack_start (GTK_BOX (vbox5), checkbutton_end_word, FALSE, FALSE, 0);
 
   // Set active or not.
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_end_word), mysession->search_end_word_match);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_end_word), session.search_end_word_match());
 
   table2 = gtk_table_new (2, 2, FALSE);
   gtk_widget_show (table2);
@@ -309,7 +317,7 @@ SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session
   radiobutton_bt_default_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_bt_commentary));
 
   // Set which radio button will be active.
-  switch (session->searchbibletimetype) {
+  switch (session.searchbibletimetype()) {
     case sbttDefaultBible:
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_bt_default), true);
       break;
@@ -341,8 +349,8 @@ SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session
   modules = bibletime->getbibles();
   combobox_set_strings (combobox_bt_bibles, modules);
   if (modules.size() > 0) {
-    if (session->search_bibletime_bible.length() > 0)
-      combobox_set_string (combobox_bt_bibles, session->search_bibletime_bible);
+    if (session.search_bibletime_bible().length() > 0)
+      combobox_set_string (combobox_bt_bibles, session.search_bibletime_bible());
     else
       combobox_set_string (combobox_bt_bibles, modules[0]);
   }
@@ -350,8 +358,8 @@ SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session
   modules = bibletime->getcommentaries();
   combobox_set_strings (combobox_bt_commentaries, modules);
   if (modules.size() > 0) {
-    if (session->search_bibletime_commentary.length() > 0)
-      combobox_set_string (combobox_bt_commentaries, session->search_bibletime_commentary);
+    if (session.search_bibletime_commentary().length() > 0)
+      combobox_set_string (combobox_bt_commentaries, session.search_bibletime_commentary());
     else
       combobox_set_string (combobox_bt_commentaries, modules[0]);
   }
@@ -394,7 +402,7 @@ SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session
 
   // Set selected value in combobox.
   int index = 0;
-  switch (session->searchresultstype) {
+  switch (session.searchresultstype()) {
     case sstLoad:
       index = 0;
       break;
@@ -472,21 +480,21 @@ SearchSpecialDialog::SearchSpecialDialog (Configuration * configuration, Session
   gtk_window_add_accel_group (GTK_WINDOW (searchspecialdialog), accel_group);
 
   // Select books feature.
-  Scripture scripture (myconfiguration->project);
-  for (unsigned int i = 0; i < scripture.books.size (); i++) {
-    selectable_books.push_back (scripture.books[i]);
+  vector <ustring> scripture_books = project_get_books (genconfig.project());
+  for (unsigned int i = 0; i < scripture_books.size (); i++) {
+    selectable_books.push_back (scripture_books[i]);
   }
 
   // Entry completion
-  completion_setup (word_entry, cpSearch, session);
+  completion_setup (word_entry, cpSearch);
   
   set_gui ();
   
   // Set dialogbox to either simple or advanced search or ....
-  if (session->search_page == 0) {
-    on_mode_change (session->search_page);
+  if (session.search_page() == 0) {
+    on_mode_change (session.search_page());
   } else {
-    for (int i = 0; i < session->search_page; i++)
+    for (int i = 0; i < session.search_page(); i++)
       on_control_f ();
   }
   
@@ -523,6 +531,7 @@ void SearchSpecialDialog::on_word_entry_changed (GtkEditable * editable, gpointe
 
 void SearchSpecialDialog::set_gui ()
 {
+  GeneralConfiguration genconfig (0);
   bool findbuttonsensitive;
   string searchword = gtk_entry_get_text (GTK_ENTRY (word_entry));
   findbuttonsensitive = (searchword.size () > 0);
@@ -530,10 +539,14 @@ void SearchSpecialDialog::set_gui ()
     if (!mybibletime->connected)
       findbuttonsensitive = false;
   gtk_widget_set_sensitive (findbutton, findbuttonsensitive);
-  gtk_widget_set_sensitive (button_books, !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_current_book)));
-  ustring message = book_selection_information (mysession, myconfiguration->project);
+  bool searchcurrentbook = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_current_book));
+  gtk_widget_set_sensitive (button_books, !searchcurrentbook);
+  if (!searchcurrentbook)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_current_chapter), false);
+  gtk_widget_set_sensitive (checkbutton_current_chapter, searchcurrentbook);
+  ustring message = book_selection_information (genconfig.project());
   gtk_label_set_text (GTK_LABEL (label_book), message.c_str());
-  gtk_label_set_text (GTK_LABEL (label_area), area_information (mysession).c_str());
+  gtk_label_set_text (GTK_LABEL (label_area), area_information ().c_str());
 }
 
 
@@ -545,39 +558,53 @@ void SearchSpecialDialog::on_findbutton_clicked (GtkButton * button, gpointer us
 
 void SearchSpecialDialog::on_find ()
 {
-  mysession->searchword = gtk_entry_get_text (GTK_ENTRY (word_entry));
-  mysession->search_case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_casesensitive));
-  mysession->search_current_book = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_current_book));
-  mysession->search_globbing = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_glob));
-  mysession->search_start_word_match = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_start_word));
-  mysession->search_end_word_match = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_end_word));
-  mysession->search_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_mode));
+  session.searchword (gtk_entry_get_text (GTK_ENTRY (word_entry)));
+  session.search_case_sensitive (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_casesensitive)));
+  session.search_current_book (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_current_book)));
+  session.search_current_chapter (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_current_chapter)));
+  session.search_globbing (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_glob)));
+  session.search_start_word_match (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_start_word)));
+  session.search_end_word_match (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_end_word)));
+  session.search_page (gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_mode)));
   ustring type = combobox_get_active_string (combobox_results);
   if (type == RESULTS_LOAD) 
-    mysession->searchresultstype = sstLoad;
+    session.searchresultstype (sstLoad);
   if (type == RESULTS_ADD) 
-    mysession->searchresultstype = sstAdd;
+    session.searchresultstype (sstAdd);
   if (type == RESULTS_REMOVE) 
-    mysession->searchresultstype = sstSubtract;
+    session.searchresultstype (sstSubtract);
   if (type == RESULTS_SHARE) 
-    mysession->searchresultstype = sstShare;
-  switch (mysession->searchresultstype) {
-    case sstLoad: mysession->highlights_clear(); // Fall through, all the way to the bottom.
+    session.searchresultstype (sstShare);
+  switch (session.searchresultstype()) {
+    case sstLoad: session.highlights_clear(); // Fall through, all the way to the bottom.
     case sstAdd:
     case sstSubtract:
-    case sstShare: mysession->highlights_add (mysession->searchword, mysession->search_case_sensitive, mysession->search_globbing, mysession->search_start_word_match, mysession->search_end_word_match);
+    case sstShare: session.highlights_add (session.searchword(), 
+                                           session.search_case_sensitive(), 
+                                           session.search_globbing(), 
+                                           session.search_start_word_match(), 
+                                           session.search_end_word_match(), 
+                                           session.area_type(), 
+                                           session.area_id (), 
+                                           session.area_intro (), 
+                                           session.area_heading (), 
+                                           session.area_chapter (), 
+                                           session.area_study (), 
+                                           session.area_notes (), 
+                                           session.area_xref (), 
+                                           session.area_verse ());
   }
-  completion_finish (word_entry, cpSearch, mysession);
+  completion_finish (word_entry, cpSearch);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_bt_default)))
-    mysession->searchbibletimetype = sbttDefaultBible;
+    session.searchbibletimetype (sbttDefaultBible);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_bt_open)))
-    mysession->searchbibletimetype = sbttOpenModules;
+    session.searchbibletimetype (sbttOpenModules);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_bt_bible)))
-    mysession->searchbibletimetype = sbttBible;
+    session.searchbibletimetype (sbttBible);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_bt_commentary)))
-    mysession->searchbibletimetype = sbttCommentary;
-  mysession->search_bibletime_bible = combobox_get_active_string (combobox_bt_bibles);
-  mysession->search_bibletime_commentary = combobox_get_active_string (combobox_bt_commentaries);
+    session.searchbibletimetype (sbttCommentary);
+  session.search_bibletime_bible (combobox_get_active_string (combobox_bt_bibles));
+  session.search_bibletime_commentary (combobox_get_active_string (combobox_bt_commentaries));
 }
 
 
@@ -589,8 +616,11 @@ void SearchSpecialDialog::on_button_books_clicked (GtkButton *button, gpointer u
 
 void SearchSpecialDialog::on_books ()
 {
-  SelectBooksDialog dialog (sbSearch, selectable_books, mysession->selected_books, myconfiguration, mysession);
-  dialog.run ();
+  set <ustring> selection = session.selected_books();
+  SelectBooksDialog dialog (sbSearch, selectable_books, selection);
+  if (dialog.run () == GTK_RESPONSE_OK) {
+    session.selected_books (selection);
+  }
   set_gui();
 }
 
@@ -609,7 +639,7 @@ void SearchSpecialDialog::on_button_area_clicked (GtkButton *button, gpointer us
 
 void SearchSpecialDialog::on_button_area ()
 {
-  AreaDialog dialog (mysession);
+  AreaDialog dialog (0);
   dialog.run ();
   set_gui ();
 }
