@@ -21,16 +21,15 @@
 #include "libraries.h"
 #include "dialogselectbooks.h"
 #include "bible.h"
+#include "books.h"
+#include "session.h"
 
 
 SelectBooksDialog::SelectBooksDialog (SelectBookDialogActions action,
-                                      const vector <ustring> &selectablebooks,
-                                      set <ustring> &selection, Configuration * configuration, Session * session)
+                                      const vector <ustring>& selectablebooks,
+                                      set <ustring>& selection):
+  genconfig (0)                                      
 {
-  // Save variables.
-  myconfiguration = configuration;
-  mysession = session;
-  
   // Build dialog.  
   selectbooksdialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (selectbooksdialog), "Select books");
@@ -53,38 +52,48 @@ SelectBooksDialog::SelectBooksDialog (SelectBookDialogActions action,
   gtk_box_pack_start (GTK_BOX (hbox2), hbox3, TRUE, TRUE, 0);
 
   // Produce six sets of the books to be displayed in the six columns.
-  unsigned int lastbook;
-  unsigned int firstbook;
-  firstbook = 0;
-  lastbook = english_id_to_index ("Joshua");
+  unsigned int amount = selectablebooks.size();
+  unsigned int amount1 = (amount / 6);
+  unsigned int amount2 = (amount / 6);
+  unsigned int amount3 = (amount / 6);
+  unsigned int amount4 = (amount / 6);
+  unsigned int amount5 = (amount / 6);
+  unsigned int amount6 = (amount / 6);
+  unsigned int remainder = amount % 6;
+  switch (remainder) {
+    case 5: amount5++;
+    case 4: amount4++; 
+    case 3: amount3++;
+    case 2: amount2++;
+    case 1: amount1++;
+  }
+  unsigned int limit1 = amount1;
+  unsigned int limit2 = limit1 + amount2;
+  unsigned int limit3 = limit2 + amount3;
+  unsigned int limit4 = limit3 + amount4;
+  unsigned int limit5 = limit4 + amount5;
+  unsigned int limit6 = limit5 + amount6;
   set <ustring> set1;
-  for (unsigned int i = firstbook; i < lastbook; i++)
-    set1.insert (id_to_biblebook_english (index_to_paratext_id (i)));
-  firstbook = lastbook;
-  lastbook = english_id_to_index ("Isaiah");
   set <ustring> set2;
-  for (unsigned int i = firstbook; i < lastbook; i++)
-    set2.insert (id_to_biblebook_english (index_to_paratext_id (i)));
-  firstbook = lastbook;
-  lastbook = english_id_to_index ("Matthew");
   set <ustring> set3;
-  for (unsigned int i = firstbook; i < lastbook; i++)
-    set3.insert (id_to_biblebook_english (index_to_paratext_id (i)));
-  firstbook = lastbook;
-  lastbook = english_id_to_index ("Romans");
   set <ustring> set4;
-  for (unsigned int i = firstbook; i < lastbook; i++)
-    set4.insert (id_to_biblebook_english (index_to_paratext_id (i)));
-  firstbook = lastbook;
-  lastbook = english_id_to_index ("James");
   set <ustring> set5;
-  for (unsigned int i = firstbook; i < lastbook; i++)
-    set5.insert (id_to_biblebook_english (index_to_paratext_id (i)));
-  firstbook = lastbook;
-  lastbook = english_id_to_index ("Revelation") + 1;
   set <ustring> set6;
-  for (unsigned int i = firstbook; i < lastbook; i++)
-    set6.insert (id_to_biblebook_english (index_to_paratext_id (i)));
+  for (unsigned int i = 0; i < amount; i++) {
+    if (i < limit1) {
+      set1.insert (selectablebooks[i]);
+    } else if (i < limit2) {
+      set2.insert (selectablebooks[i]);
+    } else if (i < limit3) {
+      set3.insert (selectablebooks[i]);
+    } else if (i < limit4) {
+      set4.insert (selectablebooks[i]);
+    } else if (i < limit5) {
+      set5.insert (selectablebooks[i]);
+    } else if (i < limit6) {
+      set6.insert (selectablebooks[i]);
+    }
+  }
 
   // The iterator goes through all models.
   GtkTreeIter iter;
@@ -472,23 +481,35 @@ void SelectBooksDialog::on_none_clicked ()
 
 void SelectBooksDialog::on_ot_clicked ()
 {
-  gtk_tree_selection_select_all (select1);
-  gtk_tree_selection_select_all (select2);
-  gtk_tree_selection_select_all (select3);
-  gtk_tree_selection_unselect_all (select4);
-  gtk_tree_selection_unselect_all (select5);
-  gtk_tree_selection_unselect_all (select6);
+  set<ustring> selection;
+  for (unsigned int i = BOOKS_OT_FIRST_ID; i <= BOOKS_OT_LAST_ID; i++) {
+    ustring book = books_id_to_english (i);
+    if (!book.empty())
+      selection.insert (book);
+  }
+  select_books (&selection, GTK_TREE_MODEL (store1), select1, treeview1);
+  select_books (&selection, GTK_TREE_MODEL (store2), select2, treeview2);
+  select_books (&selection, GTK_TREE_MODEL (store3), select3, treeview3);
+  select_books (&selection, GTK_TREE_MODEL (store4), select4, treeview4);
+  select_books (&selection, GTK_TREE_MODEL (store5), select5, treeview5);
+  select_books (&selection, GTK_TREE_MODEL (store6), select6, treeview6);
 }
 
 
-void SelectBooksDialog::on_nt_clicked ()
+void SelectBooksDialog::on_nt_clicked () 
 {
-  gtk_tree_selection_unselect_all (select1);
-  gtk_tree_selection_unselect_all (select2);
-  gtk_tree_selection_unselect_all (select3);
-  gtk_tree_selection_select_all (select4);
-  gtk_tree_selection_select_all (select5);
-  gtk_tree_selection_select_all (select6);
+  set<ustring> selection;
+  for (unsigned int i = BOOKS_NT_FIRST_ID; i <= BOOKS_NT_LAST_ID; i++) {
+    ustring book = books_id_to_english (i);
+    if (!book.empty())
+      selection.insert (book);
+  }
+  select_books (&selection, GTK_TREE_MODEL (store1), select1, treeview1);
+  select_books (&selection, GTK_TREE_MODEL (store2), select2, treeview2);
+  select_books (&selection, GTK_TREE_MODEL (store3), select3, treeview3);
+  select_books (&selection, GTK_TREE_MODEL (store4), select4, treeview4);
+  select_books (&selection, GTK_TREE_MODEL (store5), select5, treeview5);
+  select_books (&selection, GTK_TREE_MODEL (store6), select6, treeview6);
 }
 
 
@@ -505,9 +526,8 @@ void SelectBooksDialog::on_all_clicked ()
 
 void SelectBooksDialog::on_current_clicked ()
 {
-  // Select the current one.
   set<ustring> currentselection;
-  currentselection.insert (myconfiguration->book_opened);
+  currentselection.insert (genconfig.book());
   select_books (&currentselection, GTK_TREE_MODEL (store1), select1, treeview1);
   select_books (&currentselection, GTK_TREE_MODEL (store2), select2, treeview2);
   select_books (&currentselection, GTK_TREE_MODEL (store3), select3, treeview3);
