@@ -24,11 +24,11 @@
 #include "utilities.h"
 #include "stylesheetutils.h"
 #include "dialogentry.h"
-#include "project.h"
-#include "projects.h"
+#include "projectutils.h"
 #include "directories.h"
 #include "usfm.h"
 #include "gtkwrappers.h"
+#include "constants.h"
 
 
 /*
