@@ -25,10 +25,12 @@
 #include "editor.h"
 #include "highlight.h"
 #include "chapter2.h"
-#include "project.h"
 #include "usfm.h"
 #include "editornote.h"
 #include <gdk/gdkkeysyms.h>
+#include "projectutils.h"
+#include "generalconfig.h"
+#include "projectconfig.h"
 
 
 Editor::Editor (int dummy)
@@ -66,7 +68,7 @@ void Editor::textview_notes_set (GtkWidget * textview)
 }
 
 
-void Editor::chapter_load (Configuration * configuration, Book * book, unsigned int number, vector<ustring> * lines)
+void Editor::chapter_load (unsigned int number, vector<ustring> * lines)
 // Loads chapter number in the editor if lines is NULL, else loads from lines.
 {
   // Reset id related variable. A text tag in the editor gets an id number
@@ -77,19 +79,24 @@ void Editor::chapter_load (Configuration * configuration, Book * book, unsigned
   editorobjects.clear();
   EditorObject zero_object (eotNone);
   editorobjects.push_back (zero_object);
+
+  // Configuration
+  GeneralConfiguration genconfig (0);
   
   // Load text.
   if (lines) {
     load_text (* lines);
   } else {
+    myproject = genconfig.project ();
+    mybook = genconfig.book ();
     vector <ustring> mylines;
-    book->get_chapter (number, mylines);
-    load_text (mylines);    
+    mylines = project_retrieve_chapter (myproject, mybook, number);
+    load_text (mylines);
   }
  
   // Format screen.
   gtk_text_buffer_set_text (textbuffer_notes, "", -1);
-  format (configuration);
+  format ();
   
   // Set the editors ummodified.
   gtk_text_buffer_set_modified (textbuffer_text, false);
@@ -110,7 +117,7 @@ void Editor::load_text (vector<ustring> & lines)
   gtk_text_buffer_remove_all_tags (textbuffer_text, &startiter, &enditer);
   
   // Prepare for tagging the versenumbers.
-  Highlight highlight (textbuffer_text);
+  Highlight highlight (textbuffer_text, textview_text);
 
   // Set the new text.
   Chapter2 chapter2 (lines);
@@ -127,7 +134,7 @@ void Editor::load_text (vector<ustring> & lines)
 }
 
 
-void Editor::format_notes (Configuration * configuration)
+void Editor::format_notes ()
 /*
 Formats the bible notes: footnotes, endnotes, crossreferences.
 This function may be called with an empty notes buffer, but also with a full one.
@@ -140,6 +147,9 @@ Therefore we need to set up a strategy for how to deal with this issue.
   any raw code.
 */
 {
+  // Configuration data.
+  GeneralConfiguration genconfig (0);
+  
   {
     // Store current note text.
     map<int, ustring> current_notes = text_note_get_all ();
@@ -184,11 +194,11 @@ Therefore we need to set up a strategy for how to deal with this issue.
   }
 
   // Usfm data.
-  Project project (configuration->project);
-  Usfm usfm (project.get_stylesheet());
+  ProjectConfiguration projectconfig (genconfig.project ());
+  Usfm usfm (projectconfig.stylesheet());
 
   // Prepare for the notes.
-  EditorNote editornote (usfm, configuration);
+  EditorNote editornote (usfm);
 
   // Go through each line of the textbuffer.
   int number_of_lines = gtk_text_buffer_get_line_count (textbuffer_text);
@@ -257,11 +267,8 @@ Therefore we need to set up a strategy for how to deal with this issue.
         infotag = gtk_text_buffer_create_tag (textbuffer_notes, NULL, 
           "editable", false, "scale", PANGO_SCALE_SMALL, "foreground", "gray", NULL);
         gtk_text_buffer_get_end_iter (textbuffer_notes, &iter);
-        ustring infotext = " (";
-        if (editornote.types[i2] == eotFootnote) infotext.append ("footnote");
-        if (editornote.types[i2] == eotEndnote) infotext.append ("endnote");
-        if (editornote.types[i2] == eotCrossreference) infotext.append ("crossreference");
-        infotext.append (")\n");
+        ustring infotext = end_note_signature (editornote.types[i2]);
+        infotext.append ("\n");
         gtk_text_buffer_insert_with_tags (textbuffer_notes, &iter, infotext.c_str(), -1, infotag, NULL);
       }
     }
@@ -282,36 +289,27 @@ Therefore we need to set up a strategy for how to deal with this issue.
 }
 
 
-void Editor::chapter_save (Configuration * configuration, Book * book, Indexer * indexer)
+void Editor::chapter_save ()
 {
-  // See whether it was changed.
+  // If text wasn't changed, bail out.
   bool buffer_modified = gtk_text_buffer_get_modified (textbuffer_text);
   if (gtk_text_buffer_get_modified (textbuffer_notes))
     buffer_modified = true;
-  if (buffer_modified) {
-    // Get chapter text.   
-    GtkTextIter startiter, enditer;
-    gtk_text_buffer_get_start_iter (textbuffer_text, &startiter);
-    gtk_text_buffer_get_end_iter (textbuffer_text, &enditer);
-    ustring chaptertext = text_get (&startiter, &enditer);
-    // Clean it up a bit.
-    chaptertext = trim (chaptertext) + "\n";
-    // Divide it into lines.    
-    vector <ustring> lines;
-    size_t position = chaptertext.find ("\n");
-    while (position != string::npos) {
-      lines.push_back (chaptertext.substr (0, position));
-      chaptertext.erase (0, ++position);
-      position = chaptertext.find ("\n");
-    }
-    // Put the lines back in the chapter and save the whole book.
-    book->set_chapter (lines);
-    book->save ();
-    // Set the buffer to unmodified.
-    gtk_text_buffer_set_modified (textbuffer_text, false);
-    // (Re-)index this chapter.
-    indexer->update_chapter (configuration->project, book->get_id_index (), book->get_previously_presented_chapter (), lines);
-  }
+  if (!buffer_modified) 
+    return;
+  // Get chapter text.   
+  GtkTextIter startiter, enditer;
+  gtk_text_buffer_get_start_iter (textbuffer_text, &startiter);
+  gtk_text_buffer_get_end_iter (textbuffer_text, &enditer);
+  ustring chaptertext = text_get (&startiter, &enditer);
+  // Clean it up a bit and divide it into lines.
+  ParseLine parseline (trim (chaptertext));
+  // Add verse information.
+  CategorizeChapterVerse ccv (parseline.lines);
+  // Store chapter in database.
+  project_store_chapter (myproject, mybook, ccv);
+  // Set the buffer to unmodified.
+  gtk_text_buffer_set_modified (textbuffer_text, false);
 }
 
 
@@ -400,7 +398,7 @@ void Editor::footnote_body_scroll_to ()
 }
 
 
-bool Editor::footnote_caller_remove (Configuration * configuration, bool backspace)
+bool Editor::footnote_caller_remove (bool backspace)
 // Remove possible footnote caller we are at, and the associated footnote body.
 // If BackSpace was pressed, remove a caller before the insertion point.
 // This function is called by the keyboard handler, and returns true if a footnote
@@ -454,7 +452,7 @@ bool Editor::footnote_caller_remove (Configuration * configuration, bool backspa
   }
   // If something was deleted, reformat the screen.
   if (footnote_deleted)
-    format (configuration);
+    format ();
   // Returnvalue shows whether we deleted a note.
   return footnote_deleted;
 }
@@ -553,7 +551,11 @@ ustring Editor::text_note_get (gint id)
   text.append (caller);
   text.append (" ");
   // Extract the text from the notes textview.
+  // There is a problem that text, added at the very end of a note, does not
+  // have the same tag applied as the existing note text. Solve that by 
+  // iterating beyond the note, till a certain signature is encountered.
   ustring notetext;
+  bool was_right_id = false;
   GtkTextIter iter1, iter2;
   gtk_text_buffer_get_start_iter (textbuffer_notes, &iter2);
   iter1 = iter2;
@@ -565,8 +567,11 @@ ustring Editor::text_note_get (gint id)
       GtkTextTag *tag = (GtkTextTag *) tagp->data;
       gint id2 = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tag), "id"));
       if (id2 == id) {
-        notetext.append (gtk_text_iter_get_text (&iter1, &iter2));
-      }
+        was_right_id = true;
+      } 
+    }
+    if (was_right_id) {
+      notetext.append (gtk_text_iter_get_text (&iter1, &iter2));
     }
     // Free tags list if tags were found.
     if (tags) {
@@ -578,6 +583,13 @@ ustring Editor::text_note_get (gint id)
   // Clear out first bit: caller and space.
   notetext.erase (0, notetext.find (" "));
   notetext = trim (notetext);
+  // Clear out anything after signature.
+  {
+    ustring signature = end_note_signature (editorobject.type);
+    size_t position = notetext.find (signature);
+    notetext.erase (position, 1000000);
+    notetext = trim (notetext);
+  }
   text.append (notetext);
   text.append (closer);
   // Return result.
@@ -588,7 +600,7 @@ ustring Editor::text_note_get (gint id)
 map <int, ustring> Editor::text_note_get_all ()
 // Gets a map with all note text in it.
 {
-  // Result variabele.
+  // Result variable.
   map <int, ustring> result;
   
   // Get all available ids in the notes buffer.
@@ -632,9 +644,9 @@ map <int, ustring> Editor::text_note_get_all ()
 }
 
 
-void Editor::format (Configuration * configuration)
+void Editor::format ()
 {
-  format_notes (configuration);
+  format_notes ();
   footnote_body_scroll_to ();
 }
 
@@ -688,3 +700,14 @@ It scrolls the text editor to the appropriate footnote caller.
     } 
   }
 }
+
+
+ustring Editor::end_note_signature (EditorObjectType objecttype)
+{
+  ustring signature = " (";
+  if (objecttype == eotFootnote) signature.append ("footnote");
+  if (objecttype == eotEndnote) signature.append ("endnote");
+  if (objecttype == eotCrossreference) signature.append ("crossreference");
+  signature.append (")");
+  return signature;  
+}
