@@ -21,19 +21,14 @@
 #include "libraries.h"
 #include "utilities.h"
 #include "export_utils.h"
-#include "scripture.h"
-#include "book.h"
 #include "bible.h"
 #include "usfm.h"
 #include "usfmtools.h"
-#include "project.h"
 #include "notes_utils.h"
 #include "date_time_utils.h"
 #include "export_translation_notes.h"
-#include "bookmetrics.h"
 #include "constants.h"
 #include "gwrappers.h"
-#include "categorize_chapter.h"
 #include "dialogsword.h"
 #include "directories.h"
 #include "unixwrappers.h"
@@ -44,9 +39,32 @@
 #include "languages.h"
 #include "osis.h"
 #include "mapping.h"
+#include "projectutils.h"
+#include "books.h"
+#include "progresswindow.h"
+#include "generalconfig.h"
+#include "projectconfig.h"
+#include "session.h"
 
 
-void export_to_bibleworks (Configuration * configuration, GtkWidget * parent, Progress * progress)
+void export_to_usfm (GtkWidget * parent)
+// Export the whole project to USFM files.
+{
+  GeneralConfiguration genconfig (0);
+  Session session (0);
+  ustring directory = gtkw_file_chooser_select_folder (parent, "Select the folder where to store the files", session.export_usfm_directory());
+  if (!directory.empty()) {
+    session.export_usfm_directory (directory);
+    ustring command = "bibledit --export-project-usfm --gui ";
+    command.append ("--project" + shell_quote_space (genconfig.project()));
+    command.append ("--directory" + shell_quote_space (directory));
+    command.append ("&");
+    system (command.c_str());
+  }    
+}
+
+
+void export_to_bibleworks (GtkWidget * parent)
 /*
 Exports a whole project to a file that is fit for being imported by the 
 BibleWorks Version Database Compiler.
@@ -60,12 +78,14 @@ This done in this manner:
 Yes, this is a bit rough, I know...
 */
 {
+  // Configuration
+  GeneralConfiguration genconfig (0);
   // Get the filename where the data to save to.
   GtkWidget *dialog;
   dialog = gtk_file_chooser_dialog_new ("Save to a file", GTK_WINDOW (parent),
     GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
-  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), configuration->export_to_bibleworks_filename.c_str ());
+  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), genconfig.export_to_bibleworks_filename().c_str ());
   int result = gtk_dialog_run (GTK_DIALOG (dialog));
   ustring filename;
   if (result == GTK_RESPONSE_ACCEPT) 
@@ -73,54 +93,64 @@ Yes, this is a bit rough, I know...
   gtk_widget_destroy (dialog);  
   if (result == GTK_RESPONSE_ACCEPT) {
     // Store (new) filename.
-    configuration->export_to_bibleworks_filename = filename;
-    // Notebook shows progress bar.
-    progress->start();
+    genconfig.export_to_bibleworks_filename_set (filename);
+    // Progress bar.
+    ProgressWindow progresswindow ("Export", true);
     // Start process.
     try
     {
-      Project project (configuration->project);
-      Usfm usfm (project.get_stylesheet());
+      ProjectConfiguration projectconfig ("");
+      Usfm usfm (projectconfig.stylesheet());
       // Write to outputfile.
       WriteText wt (filename);
-      // Open the Scripture and go through it.
-      Scripture scripture (configuration->project);
-      progress->set_iterate (0, 1, scripture.books.size());
-      progress->set_text ("Exporting");
-      for (unsigned int bk = 0; bk < scripture.books.size(); bk++) {
+      // Get all the books and go through them.
+      vector<ustring> scripture_books = project_get_books (genconfig.project());
+      progresswindow.set_iterate (0, 1, scripture_books.size());
+      progresswindow.set_text ("Exporting");
+      for (unsigned int bk = 0; bk < scripture_books.size(); bk++) {
         // Progress information.
-        progress->iterate();
-        if (progress->cancel) {
-          progress->finish();
-          return;
-        }
+        progresswindow.iterate();
+        if (progresswindow.cancel) return;
         // Get the abbreviation to be used.
         ustring abbreviation;
-        unsigned index = english_id_to_index (scripture.books[bk]);
-        abbreviation = index_to_bibleworks_id (index);
-        // Open the book and its metrics.
-        Book book (scripture.paths[bk], false, "");
-        BookMetrics bookmetrics (scripture.paths[bk]);
+        unsigned index = books_english_to_id (scripture_books[bk]);
+        abbreviation = books_id_to_bibleworks (index);
         // Handle each chapter in the book.
-        vector<unsigned int> chapters = bookmetrics.get_chapters();
+        vector<unsigned int> chapters = project_get_chapters (genconfig.project(), scripture_books[bk]);
         for (unsigned int ch = 0; ch < chapters.size(); ch++) {
           // Do not export chapter 0.
           if (chapters[ch] == 0)
             continue;
           // Get the chapter and handle it.
           vector<ustring> lines;
-          book.get_chapter (chapters[ch], lines);
-          CategorizeChapter categorize_chapter (lines);
-          for (unsigned int vs = 0; vs < categorize_chapter.verses.size(); vs++) {
+          lines = project_retrieve_chapter (genconfig.project(), scripture_books[bk], chapters[ch]);
+          CategorizeChapterVerse ccv (lines);
+          for (unsigned int vs = 0; vs < ccv.verse.size(); vs++) {
             // Do not export verse 0.
-            if (categorize_chapter.verses[vs] == "0")
+            if (ccv.verse[vs] == "0")
               continue;
+            // Bibledit does not handle combined verses like 1-4a., etc. So make
+            // a simple verse.
+            ustring verse = number_in_string (ccv.verse[vs]);
+            // Remove the verse number from the line itself, except when the verse
+            // was not a simple one, e.g. it was something like verse 1-3.
+            size_t position = ccv.line[vs].find (" ");
+            if (position != string::npos)
+              ccv.line[vs].erase (0, ++position);
+            if (verse == ccv.verse[vs]) {
+              position = ccv.line[vs].find (" ");
+              if (position != string::npos)
+                ccv.line[vs].erase (0, ++position);
+            }
+            CategorizeLine cat_line (ccv.line [vs]);
             // Output verse.
             wt.text(abbreviation);
             wt.text(" ");
             wt.text(convert_to_string (chapters[ch]));
             wt.text(":");
-            wt.text(categorize_chapter.verse_texts[vs]);
+            wt.text(verse);
+            wt.text(" ");
+            wt.text(cat_line.verse);
             wt.text("\n");
           }
         }
@@ -131,58 +161,71 @@ Yes, this is a bit rough, I know...
       cerr << "Converting to BibleWorks: " << ex.what () << endl;
     }
   }
-  // Notebook hides progress bar.
-  progress->finish();
 }
 
 
-void export_translation_notes (const ustring& filename, ExportNotesFormat format, const vector<gint> & ids_to_display, bool export_all, GtkWidget * parent, Progress * progress)
+void export_translation_notes (const ustring& filename, ExportNotesFormat format, const vector<unsigned int> & ids_to_display, bool export_all, GtkWidget * parent)
 {
-  ExportTranslationNotes etn (filename, format, ids_to_display, export_all, parent, progress);
+  ExportTranslationNotes etn (filename, format, ids_to_display, export_all, parent);
 }
 
 
-void export_to_sword (Configuration * configuration, GtkWidget * parent, Progress * progress)
+void export_to_sword_interactive ()
+// Exports a whole project to a SWORD module.
+{
+  // Dialog for information entry.
+  {
+    SwordDialog sworddialog (0);
+    if (sworddialog.run() != GTK_RESPONSE_OK)
+      return;
+  }
+  GeneralConfiguration genconfig (0);
+  export_to_sword_script (genconfig.project(), "", true);
+}
+
+
+void export_to_sword_script (const ustring& project, ustring directory, bool gui)
 /*
 Exports a whole project to a SWORD module.
 At the time of writing this, the information on how to create a module for 
 sword was found at http://www.crosswire.org/sword/develop/swordmodule/
 Here's how we do the conversion
 - A .conf file is created and put in ~/.sword/mods.d/
-- We use the "osis" format as this offers most options.
+- We use the "osis" format as this offers most options and seems best supported.
 - Create the module with program osis2mod.
 - The text will be stored in ~/.sword/modules/texts/bibledit/<name>/
 */
 {
-  // Dialog for information entry.
-  {
-    SwordDialog sworddialog (configuration);
-    if (sworddialog.run() != GTK_RESPONSE_OK)
-      return;
-  }
   // Check for converter.
   if (!unix_which ("osis2mod")) {
-    gtkw_dialog_error (NULL, "The SWORD compiler osis2mod was not found.");
+    ustring message = "The SWORD compiler osis2mod was not found.";
+    if (gui) gtkw_dialog_error (NULL, message);
+    cerr << message << endl;
     return;
   }
-  // Notebook shows progress bar.
-  progress->start();
-  progress->set_text ("Collecting data");
+  // Directory to store module.
+  if (directory.empty())
+    directory = g_get_home_dir();
+  // Configuration
+  GeneralConfiguration genconfig (0);
+  // Progress information.
+  ProgressWindow * progresswindow = NULL;
+  if (gui) progresswindow = new ProgressWindow ("Exporting project", true);
   // Open the project that contains the data.
-  Project project (configuration);
+  ProjectConfiguration projectconfig (project);
   // The temporal directories for the data.
   ustring base_directory = gw_build_filename (directories_get_temp(), "sword");
   remove_directory (base_directory);
   create_directory (base_directory);
   ustring absolute_conf_directory = gw_build_filename (base_directory, "mods.d");
   create_directory (absolute_conf_directory);
-  ustring relative_text_directory = gw_build_filename ("modules", "texts", "bibledit", lowerCase (project.sword_name));
+  ustring relative_text_directory = gw_build_filename ("modules", "texts", "bibledit", lowerCase (genconfig.project() + projectconfig.sword_name()));
   ustring absolute_text_directory = gw_build_filename (base_directory, relative_text_directory);
   create_directory (absolute_text_directory);
   // Create the configuration file.
   vector<ustring> lines;
   ustring line;
-  lines.push_back ("[" + project.sword_name + "]");
+  lines.push_back ("[" + genconfig.project() + projectconfig.sword_name() + "]");
   line = "DataPath=.";
   line.append (G_DIR_SEPARATOR_S + relative_text_directory + G_DIR_SEPARATOR_S);
   lines.push_back (line);
@@ -196,48 +239,51 @@ Here's how we do the conversion
   lines.push_back ("GlobalOptionFilter=OSISHeadings");
   lines.push_back ("GlobalOptionFilter=OSISRedLetterWords");
   lines.push_back ("MinimumVersion=1.5.6");
-  lines.push_back ("Lang=" + language_encode (project.sword_language));
-  lines.push_back ("Version=" + project.sword_version);
-  lines.push_back ("Description=" + project.sword_description);
-  lines.push_back ("About=" + project.sword_about);
-  lines.push_back ("LCSH=" + project.sword_lcsh);
-  lines.push_back ("DistributionLicense=" + project.sword_license);
-  write_lines (gw_build_filename (absolute_conf_directory, lowerCase (project.sword_name) + ".conf"), lines);
+  lines.push_back ("Lang=" + language_encode (projectconfig.sword_language()));
+  lines.push_back ("Version=" + projectconfig.sword_version());
+  lines.push_back ("Description=" + genconfig.project() + projectconfig.sword_description());
+  lines.push_back ("About=" + genconfig.project() + projectconfig.sword_about());
+  lines.push_back ("LCSH=" + projectconfig.sword_lcsh());
+  lines.push_back ("DistributionLicense=" + projectconfig.sword_license());
+  write_lines (gw_build_filename (absolute_conf_directory, lowerCase (genconfig.project() + projectconfig.sword_name()) + ".conf"), lines);
   lines.clear();
   // Start process of producing the text file.
   ustring inputfile;
   try
   {
     // Prepare for notes and inline text.
-    Usfm usfm (project.get_stylesheet());
-    SwordNote swordnote (usfm, configuration, true);
+    Usfm usfm (projectconfig.stylesheet());
+    SwordNote swordnote (usfm, true);
     UsfmInlineMarkers usfm_inline_markers (usfm);
     // Write to inputfile.
     inputfile = gw_build_filename (directories_get_temp(), "sword_osis_input.txt");
     unlink (inputfile.c_str());
     WriteText wt (inputfile);
     // Write out xml headers.
-    OsisRoot osisroot (&wt, project.sword_name, project.sword_description);
-    // Open the Scripture and go through it.
-    Scripture scripture (configuration->project);
-    progress->set_iterate (0, 1, scripture.books.size());
-    for (unsigned int bk = 0; bk < scripture.books.size(); bk++) {
+    OsisRoot osisroot (&wt, genconfig.project() + projectconfig.sword_name(), genconfig.project() + projectconfig.sword_description());
+    // Get all the books and go through them.
+    vector <ustring> scripture_books = project_get_books (genconfig.project());
+    if (gui)
+      progresswindow->set_iterate (0, 1, scripture_books.size());
+    for (unsigned int bk = 0; bk < scripture_books.size(); bk++) {
       // Progress information.
-      progress->iterate();
-      if (progress->cancel) {
-        progress->finish();
-        return;
+      if (gui) {
+        progresswindow->iterate();
+        progresswindow->set_text (scripture_books[bk]);
+        if (progresswindow->cancel) {
+          delete progresswindow;
+          return;
+        }
       }
+      cout << scripture_books[bk] << endl;
       // Signal "new book" to notes system.
       swordnote.new_book ();
       // Open book in osis code.
-      OsisBook osisbook (&wt, scripture.books[bk]);
-      // Open the book and its metrics.
-      Book book (scripture.paths[bk], false, "");
-      // Verse mapping. SWORD's versification is according to the King James Bible,
-      // called "English" in our system.
-      Mapping mymapping (project.versification, scripture.books[bk]);
-      Mapping swordmapping ("English", scripture.books[bk]);
+      OsisBook osisbook (&wt, scripture_books[bk]);
+      // Verse mapping. SWORD's versification is according to the King James 
+      // Bible, called "English" in our system.
+      Mapping mymapping (projectconfig.versification(), scripture_books[bk]);
+      Mapping swordmapping ("English", scripture_books[bk]);
       // Go through the book and remap verses, if need be.
       vector<unsigned int> remapped_chapters;
       vector<ustring> remapped_verses;
@@ -247,15 +293,16 @@ Here's how we do the conversion
         ustring swordverse = "0";
         int mychapter = 0;
         int swordchapter = 0;
-        for (unsigned int i2 = 0; i2 < book.cleaned_lines.size(); i2++) {
-          ustring line = book.cleaned_lines[i2];
+        vector <ustring> book_lines = project_retrieve_book (genconfig.project(), scripture_books[bk]);
+        for (unsigned int i2 = 0; i2 < book_lines.size(); i2++) {
+          ustring line = book_lines[i2];
           ustring marker = usfm_extract_marker (line);
           if (usfm.is_chapter_number (marker)) {
             mychapter = convert_to_int (number_in_string (line));
           } else if (usfm.is_verse_number (marker)) {
             // Extract verse number.
             size_t position = line.find (" ");
-            position = CLAMP (position, 0, line.length() - 1);
+            position = CLAMP (position, 0, line.length());
             myverse = line.substr (0, position);
             // Erase verse number from the line.
             position++;
@@ -293,7 +340,7 @@ Here's how we do the conversion
             remapped_chapters.push_back (swordchapter);
             remapped_verses.push_back (swordverse);
             // Store line with marker in it. Needed for formatting.
-            remapped_lines.push_back (book.cleaned_lines[i2]);
+            remapped_lines.push_back (book_lines[i2]);
           }
         } 
       }
@@ -407,21 +454,21 @@ Here's how we do the conversion
   {
     cerr << "Export: " << ex.what () << endl;
   }
+  // Hide progress.
+  if (progresswindow) delete progresswindow;
   // Convert the inputfile using the sword api utility.
   ustring command;
   command = "osis2mod " + absolute_text_directory + " " + inputfile;
-  run_shell_progress (command, "osis2mod", progress, "Compiling", true, 3000000);
+  run_shell_progress (command, "osis2mod", gui, "Compiling", true, 3000000);
   // Install it.
-  command = "cp -r " + gw_build_filename (base_directory, "*") + " " + configuration->export_to_sword_install_path;
+  command = "cp -r " + gw_build_filename (base_directory, "*") + " " + genconfig.export_to_sword_install_path();
   system (command.c_str());
   // Compress and save the module.
-  ustring zipfile = temporary_file (project.sword_name) + ".zip";
+  ustring zipfile = temporary_file (genconfig.project() + projectconfig.sword_name()) + ".zip";
   unlink (zipfile.c_str());
   command = "cd" + shell_quote_space (base_directory) + "; ";
   command.append ("zip -r" + shell_quote_space (zipfile) + "*");
   system (command.c_str());
-  command = "mv" + shell_quote_space (zipfile) + shell_quote_space (configuration->export_to_sword_module_path);
+  command = "mv" + shell_quote_space (zipfile) + shell_quote_space (genconfig.export_to_sword_module_path());
   system (command.c_str());
-  // Notebook hides progress bar.
-  progress->finish();
 }
