@@ -24,16 +24,16 @@
 
 #include "libraries.h"
 #include <gtk/gtk.h>
-#include "configuration.h"
-#include "progress.h"
 
 
 enum ExportNotesFormat {BibleditVersion3, ScriptureNotesVersion20};
 
 
-void export_to_bibleworks (Configuration * configuration, GtkWidget * parent, Progress * progress);
-void export_translation_notes (const ustring& filename, ExportNotesFormat format, const vector<gint> & ids_to_display, bool export_all, GtkWidget * parent, Progress * progress);
-void export_to_sword (Configuration * configuration, GtkWidget * parent, Progress * progress);
+void export_to_usfm (GtkWidget * parent);
+void export_to_bibleworks (GtkWidget * parent);
+void export_translation_notes (const ustring& filename, ExportNotesFormat format, const vector<unsigned int> & ids_to_display, bool export_all, GtkWidget * parent);
+void export_to_sword_interactive ();
+void export_to_sword_script (const ustring& project, ustring directory, bool gui);
 
 
 #endif
