@@ -23,32 +23,27 @@
 
 
 #include <gtk/gtk.h>
-#include "configuration.h"
 #include "track.h"
-#include "book.h"
-//#include "backup.h"
 #include "undo.h"
 #include "windowsoutpost.h"
-#include "scripture.h"
 #include "ustring.h"
 #include "bibletime.h"
 #include "usfm.h"
-#include "progress.h"
-#include "indexer.h"
 #include "splashscreen.h"
-#include "session.h"
 #include "navigation.h"
 #include "note_editor.h"
 #include "editorobject.h"
 #include "editor.h"
+#include "generalconfig.h"
+#include "session.h"
 
 
 class MainWindow
 {
 public:
   MainWindow (SplashScreen * splashscreen);
-  virtual ~ MainWindow ();
-  virtual int run ();
+  ~ MainWindow ();
+  int run ();
   GtkWidget *mainwindow;
 protected:
   /* IMPORTANT If more menu items are added see whether they should be made 
@@ -74,6 +69,8 @@ protected:
   GtkWidget *export_project;
   GtkWidget *image3298;
   GtkWidget *export_project_menu;
+  GtkWidget *export_usfm_files;
+  GtkWidget *image12814;
   GtkWidget *to_bibleworks_version_database_compiler;
   GtkWidget *image3299;
   GtkWidget *export_to_sword_module;
@@ -84,6 +81,8 @@ protected:
   GtkWidget *image2764;
   GtkWidget *parallel_bible;
   GtkWidget *image11566;
+  GtkWidget *synchronize_projects;
+  GtkWidget *image12270;
   GtkWidget *file_references;
   GtkWidget *image465;
   GtkWidget *file_references_menu;
@@ -252,11 +251,17 @@ protected:
   GtkWidget *image7238;
   GtkWidget *unwanted_words;
   GtkWidget *image7631;
+  GtkWidget *word_count_inventory;
+  GtkWidget *image13715;
   GtkWidget *check_punctuation;
   GtkWidget *image7366;
   GtkWidget *check_punctuation_menu;
   GtkWidget *check_matching_pairs;
   GtkWidget *image7367;
+  GtkWidget *menutools;
+  GtkWidget *menutools_menu;
+  GtkWidget *line_cutter_for_hebrew_text1;
+  GtkWidget *image13532;
   GtkWidget *menuitem_preferences;
   GtkWidget *menuitem_preferences_menu;
   GtkWidget *notes_preferences;
@@ -269,13 +274,17 @@ protected:
   GtkWidget *image5972;
   GtkWidget *ignored_references1;
   GtkWidget *image6467;
+  GtkWidget *prefs_books;
+  GtkWidget *image12167;
+  GtkWidget *preferences_synchronization;
+  GtkWidget *image12530;
+  GtkWidget *preferences_debug;
   GtkWidget *menuitem_help;
   GtkWidget *menuitem_help_menu;
   GtkWidget *help1;
   GtkWidget *system_log1;
   GtkWidget *image4388;
   GtkWidget *about1;
-  GtkWidget *image107;
   GtkWidget *toolbar1;
   GtkWidget *button_back;
   GtkWidget *image1;
@@ -341,14 +350,13 @@ protected:
   GtkWidget *label20;
   GtkWidget *label_notetools;
   GtkWidget *hbox5;
-  GtkWidget *notebook2;
   GtkWidget *hbox7;
   GtkWidget *statuslabel_stylesheet;
-  GtkWidget *label10;
-  GtkWidget *hbox6;
-  GtkWidget *progressbar1;
-  GtkWidget *progress_cancel_button;
-  GtkWidget *label11;
+  GtkWidget *statusbutton_attention;
+  GtkWidget *alignment1;
+  GtkWidget *hbox8;
+  GtkWidget *image12350;
+  GtkWidget *label21;
   GtkWidget *statusbar1;
   GtkAccelGroup *accel_group;
   // Gtk variables manually added.
@@ -364,380 +372,417 @@ protected:
   Editor editor;
 
   /* Initialization */
-  virtual void initialize_project (bool init_book_selection);
-  virtual void enable_or_disable_widgets (bool enable);
+  void close_open_project (ustring project);
+  void initialize_project (bool init_book_selection);
+  void enable_or_disable_widgets (bool enable);
 
   /* Menu callbacks */
   static void on_open1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void open ();
+  void open ();
   static void on_close1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void close ();
+  void close ();
   static void on_new1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void newproject ();
+  void newproject ();
   static void on_properties1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void editproject ();
+  void editproject ();
   static void on_delete1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void deleteproject ();
+  void deleteproject ();
   static void on_quit1_activate (GtkMenuItem * menuitem, gpointer user_data);
   static void on_system_log1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void viewlog ();
+  void viewlog ();
   static void on_help1_activate (GtkMenuItem * menuitem, gpointer user_data);
   static void on_about1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void showabout ();
+  void showabout ();
   static void on_undo1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void menu_undo ();
+  void menu_undo ();
   static void on_redo1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void menu_redo ();
+  void menu_redo ();
   static void on_edit1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void menu_edit ();
+  void menu_edit ();
   static void on_find_and_replace1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void menu_replace ();
+  void menu_replace ();
   static void on_findspecial1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void menu_findspecial ();
+  void menu_findspecial ();
   static void on_screen_font_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_screen_font ();
+  void on_screen_font ();
   static void on_printer_font_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_printer_font ();
+  void on_printer_font ();
   static void on_import1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void menu_import ();
+  void menu_import ();
   static gboolean on_mainwindow_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
   static gboolean on_mainwindow_focus_in_event (GtkWidget *widget, GdkEventFocus *event, gpointer user_data);
-  virtual void on_mainwindow_focus_in (GdkEventFocus *event);
+  void on_mainwindow_focus_in (GdkEventFocus *event);
   static void on_print_project_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_print_project ();
+  void on_print_project ();
   static void on_insert1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_insert ();
+  void on_menu_insert ();
   static void on_notes_preferences_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_notes_preferences ();
+  void on_notes_preferences ();
   static void on_copy_project_to_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_copy_project_to ();
+  void on_copy_project_to ();
   static void on_compare_with1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_compare_with ();
+  void on_compare_with ();
   static void on_printingprefs_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_printing_preferences ();
+  void on_printing_preferences ();
   static void on_formatting_objects_processor_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_formatter ();
-  static void on_progress_cancel_button_clicked (GtkButton *button, gpointer user_data);
-  virtual void on_progress_cancel_button ();
+  void on_formatter ();
   static void on_parallel_bible_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_parallel_bible ();
+  void on_parallel_bible ();
   static void on_screen_layout_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_screen_layout ();
+  void on_screen_layout ();
+  static void on_prefs_books_activate (GtkMenuItem *menuitem, gpointer user_data);
+  void on_prefs_books ();
+  static void on_preferences_debug_activate (GtkMenuItem *menuitem,  gpointer user_data);
+  void on_preferences_debug ();
 
   /* Navigation  */
   static void on_combo_entry_book_changed (GtkEditable * editable, gpointer user_data);
-  virtual void on_combo_book_changed ();
+  void on_combo_book_changed ();
   static void on_combo_entry_chapter_changed (GtkEditable * editable, gpointer user_data);
-  virtual void on_combo_chapter_changed ();
+  void on_combo_chapter_changed ();
   static void on_combo_entry_verse_changed (GtkEditable * editable, gpointer user_data);
-  virtual void on_combo_verse_changed ();
+  void on_combo_verse_changed ();
   static void on_next_verse_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void goto_next_verse ();
+  void goto_next_verse ();
   static void on_previous_verse_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void goto_previous_verse ();
+  void goto_previous_verse ();
   static void on_next_chapter_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void goto_next_chapter ();
+  void goto_next_chapter ();
   static void on_previous_chapter_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void goto_previous_chapter ();
+  void goto_previous_chapter ();
   static void on_next_book_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void goto_next_book ();
+  void goto_next_book ();
   static void on_previous_book_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void goto_previous_book ();
+  void goto_previous_book ();
   static void on_reference_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void goto_reference ();
-  virtual void go_to_new_reference ();
+  void goto_reference_interactive ();
+  void go_to_new_reference ();
+  bool go_to_new_reference_active; // Probably can go out.
+  guint go_to_new_reference_event_id;
+  static bool go_to_new_reference_delayer (gpointer user_data);
+  void go_to_new_reference_executer ();
   ustring editor_current_book, editor_current_chapter, editor_current_verse;
   bool settingcombos;
   static void on_textview_edit_move_cursor (GtkTextView * textview, GtkMovementStep step, gint count, gboolean extend_selection, gpointer user_data);
-  static bool mainwindow_on_cursor_moved_delayed (gpointer data);
-  virtual void on_cursor_moved ();
+  void on_cursor_moved_delayer ();
+  static bool on_cursor_moved_delayer_handler (gpointer user_data);
+  guint cursor_moved_delayer_event_id;
+  void on_cursor_moved ();
   bool focus_programmatically_being_grabbed;
   static void on_textview_edit_grab_focus (GtkWidget * widget, gpointer user_data);
-  virtual void on_grab_focus ();
+  void on_grab_focus ();
+  guint grab_focus_event_id;
   ustring cursor_get_verse_number ();
   static bool mainwindow_on_grab_focus_delayed_handler (gpointer data);
   bool on_grab_focus_delayed_handler ();
   static void on_text_area1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_text_area_activate ();
+  void on_text_area_activate ();
   static void on_goto_bible_notes_area1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_bible_notes_area_activate ();
+  void on_bible_notes_area_activate ();
   static void on_tools_area1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_tools_area_activate ();
+  void on_tools_area_activate ();
   static void on_notes_area1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_notes_area_activate ();
+  void on_notes_area_activate ();
   static void on_synchronize_other_programs2_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_synchronize_other_programs ();
+  void on_synchronize_other_programs ();
   static void on_spinbutton_book_value_changed (GtkSpinButton *spinbutton, gpointer user_data);
   static void on_spinbutton_chapter_value_changed (GtkSpinButton *spinbutton, gpointer user_data);
   static void on_spinbutton_verse_value_changed (GtkSpinButton *spinbutton, gpointer user_data);
-  virtual void on_spinbutton_book ();
-  virtual void on_spinbutton_chapter ();
-  virtual void on_spinbutton_verse ();
+  void on_spinbutton_book ();
+  void on_spinbutton_chapter ();
+  void on_spinbutton_verse ();
   int spinbutton_book_previous_value;
   int spinbutton_chapter_previous_value;
   int spinbutton_verse_previous_value;
   static bool on_program_startup_timeout (gpointer data);
-  virtual void on_program_startup ();
+  void on_program_startup ();
 
   /* Tracker */
   static void on_button_forward_clicked (GtkButton * button, gpointer user_data);
   static void on_next_reference_in_history1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void goto_next_reference ();
+  void goto_next_reference ();
   static void on_button_back_clicked (GtkButton * button, gpointer user_data);
   static void on_previous_reference_in_history1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void goto_previous_reference ();
-  virtual void set_sensitivity_of_navigator_buttons ();
+  void goto_previous_reference ();
+  void set_sensitivity_of_navigator_buttons ();
   static bool mainwindow_on_tracker_timeout (gpointer data);
-  virtual bool on_tracker_timeout ();
-  virtual void set_next_or_previous_reference (bool next);
+  bool on_tracker_timeout ();
+  void set_next_or_previous_reference (bool next);
   Track track;
 
   /* Variables for general use */
-  Configuration * configuration;
-  Session * session;
-  Scripture * scripture;
-  Indexer * indexer;
+  GeneralConfiguration * genconfig;
+  Session session;
 
   /* Loading, editing, saving */
-  Book * book;
-  virtual void editor_load_chapter ();
-  virtual void editor_position_cursor_at_verse (const ustring& cursorposition);
-  virtual void editor_save ();
+  void editor_load_chapter ();
+  void editor_position_cursor_at_verse (const ustring& cursorposition);
+  ustring editor_position_cursor_at_verse_cursorposition;
+  static bool editor_position_cursor_at_verse_postponer_handler (gpointer user_data);
+  gint editor_position_cursor_at_verse_event_id;
+  void editor_position_cursor_at_verse_executer ();
+  void editor_save ();
   static bool mainwindow_on_save_timeout (gpointer data);
-  virtual bool on_save_timeout ();
-  virtual void editor_set_font ();
+  bool on_save_timeout ();
+  void editor_set_font ();
 
   /* Clipboard */
   static void on_cut1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_cut ();
+  void on_cut ();
   static void on_copy1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_copy ();
+  void on_copy ();
   static void on_paste1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_paste ();
+  void on_paste ();
 
   /* Backups */
   //  Backup backup;
 
   /* Tools Area */
   static void on_notebook_tools_switch_page (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, gpointer user_data);
-  virtual void notebook_tools_switch_page (guint page_num);
+  void notebook_tools_switch_page (guint page_num);
   static void on_file_references_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_file_references ();
+  void on_file_references ();
 
   /* List store and reference handling */
   static gboolean on_treeview_references_key_press_event (GtkWidget * widget, GdkEventKey * event, gpointer user_data);
   static gboolean on_treeview_references_button_press_event (GtkWidget * widget, GdkEventButton * event, gpointer user_data);
-  virtual void on_list_goto ();
+  void on_list_goto ();
   static void mainwindow_selection_foreach_function (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data);
   static void on_open_references1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_open_references ();
+  void on_open_references ();
   ustring selected_reference;
   static void on_references_save_as_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_save_references ();
+  void on_save_references ();
   static void on_print_references_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_print_references ();
+  void on_print_references ();
   static void on_close_references_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_clear_references ();
+  void on_clear_references ();
   static void on_delete_references_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_delete_references ();
+  void on_delete_references ();
   static void on_references_collect_iters (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data);
   static void on_next_reference1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_next_reference ();
+  void on_next_reference ();
   static void on_previous_reference1_activate (GtkMenuItem * menuitem, gpointer user_data);
-  virtual void on_previous_reference ();
+  void on_previous_reference ();
   GtkTextTag * editor_reference_tag;
   static void on_ignored_references1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_ignored_references ();
+  void on_ignored_references ();
   static gboolean on_treeview_references_popup_menu (GtkWidget *widget, gpointer user_data);
-  virtual void treeview_references_popup_menu (GtkWidget *widget);
+  void treeview_references_popup_menu (GtkWidget *widget);
   static void on_reference_hide_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_reference_hide ();
-  virtual void show_references_popup_menu (GtkWidget *my_widget, GdkEventButton *event);
+  void on_reference_hide ();
+  void show_references_popup_menu (GtkWidget *my_widget, GdkEventButton *event);
   static gboolean on_treeview_references_move_cursor (GtkTreeView *treeview, GtkMovementStep step, gint count, gpointer user_data);
   static void on_treeview_references_cursor_changed (GtkTreeView *treeview, gpointer user_data);
-  virtual void treeview_references_display_quick_reference ();
+  void treeview_references_display_quick_reference ();
 
   /* Undo and Redo */
   Undo undo;
   static void on_textbuffer_edit_insert_text (GtkTextBuffer * textbuffer, GtkTextIter * arg1, gchar * arg2, gint arg3, gpointer user_data);
   static void on_textbuffer_edit_delete_text (GtkTextBuffer * textbuffer, GtkTextIter * arg1, GtkTextIter * arg2, gpointer user_data);
-  virtual void on_textbuffer_edit_insert_delete ();  
+  void on_textbuffer_edit_insert_delete ();  
   static bool on_undo_timeout (gpointer data);
-  virtual bool undo_timeout ();
+  bool undo_timeout ();
 
   // Bibledit Windows Outpost
   // BibleTime
   static bool mainwindow_on_external_programs_timeout (gpointer data);
-  virtual bool on_external_programs_timeout ();
+  bool on_external_programs_timeout ();
   WindowsOutpost windowsoutpost;
   static void on_reference_exchange1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_reference_exchange ();
+  void on_reference_exchange ();
   BibleTime bibletime;
   ustring bibledit_bt_previous_reference;
   ustring bibletime_previous_reference;
   static gboolean on_textview_edit_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
   static gboolean on_textview_footnotes_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
-  virtual void on_texteditor_click (GtkWidget * widget, GdkEventButton *event);
-  virtual void send_word_to_toolbox (const ustring& word);
+  void on_texteditor_click (GtkWidget * widget, GdkEventButton *event);
+  void send_word_to_toolbox (const ustring& word);
 
   /* Title bar and status bar, GUI */
-  virtual void set_titlebar (const ustring& project);
-  Progress * progress;
+  void set_titlebar (const ustring& project);
   static bool on_gui_timeout (gpointer data);
-  virtual void on_gui ();
+  void on_gui ();
 
   /* Notes editor */
   static gboolean notes_motion_notify_event (GtkWidget *text_view, GdkEventMotion *event, gpointer user_data);
-  virtual gboolean on_notes_pointer_moved (GtkWidget *text_view, GdkEventMotion *event);
-  virtual void set_cursor_if_appropriate (GtkTextView *text_view, gint x, gint y);
+  gboolean on_notes_pointer_moved (GtkWidget *text_view, GdkEventMotion *event);
+  void set_cursor_if_appropriate (GtkTextView *text_view, gint x, gint y);
   gboolean hovering_over_link;
   GdkCursor *hand_cursor;
   GdkCursor *regular_cursor;
   static gboolean visibility_notify_event (GtkWidget *text_view, GdkEventVisibility *event, gpointer user_data);
-  virtual gboolean on_visibility_notified (GtkWidget *text_view, GdkEventVisibility *event);
+  gboolean on_visibility_notified (GtkWidget *text_view, GdkEventVisibility *event);
   static gboolean notes_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data);
-  virtual gboolean on_notes_key_press_event (GtkWidget *widget, GdkEventKey *event);
+  gboolean on_notes_key_press_event (GtkWidget *widget, GdkEventKey *event);
   static void notes_event_after (GtkWidget *widget, GdkEvent *event, gpointer user_data);
-  virtual void on_notes_event_after (GtkWidget *text_view, GdkEvent *ev);
-  virtual void notes_edit_if_link (GtkWidget *text_view, GtkTextIter *iter);
+  void on_notes_event_after (GtkWidget *text_view, GdkEvent *ev);
+  void notes_edit_if_link (GtkWidget *text_view, GtkTextIter *iter);
   static void on_new_note_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_new_note ();
-  virtual void notes_delete_if_link (GtkWidget *text_view, GtkTextIter *start, GtkTextIter *end);
+  void on_new_note ();
+  void notes_delete_if_link (GtkWidget *text_view, GtkTextIter *start, GtkTextIter *end);
   static void on_delete_note_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_delete_note ();
+  void on_delete_note ();
   static void on_viewnotes_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_view_notes ();
-  virtual void notes_redisplay ();
+  void on_view_notes ();
+  void notes_redisplay ();
   static void on_find_in_notes1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void find_in_notes ();
+  void find_in_notes ();
   static void on_import_notes_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_import_notes ();
+  void on_import_notes ();
   static void on_export_notes_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_export_notes ();
+  void on_export_notes ();
   static void on_button_cancel_clicked (GtkButton *button, gpointer user_data);
-  virtual void on_notes_button_cancel ();
+  void on_notes_button_cancel ();
   static void on_button_ok_clicked (GtkButton *button, gpointer user_data);
-  virtual void on_notes_button_ok ();
-  virtual void on_notes_button_ok_cancel ();
-  virtual void notes_fill_edit_screen (int id, bool newnote);
+  void on_notes_button_ok ();
+  void on_notes_button_ok_cancel ();
+  void notes_fill_edit_screen (int id, bool newnote);
   NoteEditor * note_editor;
-  virtual void note_insert_date_and_text (const ustring& text);
+  void note_insert_date_and_text (const ustring& text);
   static void on_standard_text_1_activate (GtkMenuItem *menuitem, gpointer user_data);
   static void on_standard_text_2_activate (GtkMenuItem *menuitem, gpointer user_data);
   static void on_standard_text_3_activate (GtkMenuItem *menuitem, gpointer user_data);
   static void on_standard_text_4_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_insert_standard_text (GtkMenuItem *menuitem);
+  void on_insert_standard_text (GtkMenuItem *menuitem);
   static void on_current_reference1_activate (GtkMenuItem *menuitem, gpointer user_data);
   static void on_menuitem_view_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menuitem_view ();
-  virtual void stop_displaying_more_notes ();
+  void on_menuitem_view ();
+  void stop_displaying_more_notes ();
   static bool on_display_notes_timeout (gpointer data);
-  virtual bool on_display_notes ();
+  bool on_display_notes ();
   unsigned int on_display_notes_timer;
   unsigned int notes_ids_to_display_pointer;
-  vector<gint> notes_ids_to_display;
+  vector<unsigned int> notes_ids_to_display;
   static bool on_note_display_one (gpointer data);
-  virtual bool note_display_one ();
+  bool note_display_one ();
   static void on_get_references_from_note_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_get_references_from_note ();
-  virtual void notes_get_references_from_link (GtkWidget *text_view, GtkTextIter *iter);
+  void on_get_references_from_note ();
+  void notes_get_references_from_link (GtkWidget *text_view, GtkTextIter *iter);
   
   /* Export */
+  static void on_export_usfm_files_activate (GtkMenuItem *menuitem, gpointer user_data);
+  void on_export_usfm_files ();
   static void on_to_bibleworks_version_compiler_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_to_bibleworks_version_compiler ();
+  void on_to_bibleworks_version_compiler ();
   static void on_export_to_sword_module_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_export_to_sword_module ();
+  void on_export_to_sword_module ();
 
   /* Checks */
   static void on_check1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_check ();
+  void on_menu_check ();
   static void on_markers1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_check_markers ();
+  void on_menu_check_markers ();
   static void on_validate_usfms1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_check_markers_validate ();
+  void on_menu_check_markers_validate ();
   static void on_count_usfms1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_check_markers_count ();  
+  void on_menu_check_markers_count ();  
   static void on_compare_usfm1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_check_markers_compare ();  
+  void on_menu_check_markers_compare ();  
   static void on_chapters_and_verses1_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_check_chapters_and_verses ();
+  void on_menu_check_chapters_and_verses ();
   static void on_count_characters_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_count_characters ();
+  void on_count_characters ();
   static void on_unwanted_patterns_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_unwanted_patterns ();
+  void on_unwanted_patterns ();
   static void on_check_capitalization_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_check_capitalization ();
+  void on_check_capitalization ();
   static void on_check_repetition_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_check_repetition ();
+  void on_check_repetition ();
   static void on_check_matching_pairs_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_check_matching_pairs ();
+  void on_check_matching_pairs ();
   static void on_unwanted_words_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_unwanted_words ();
+  void on_unwanted_words ();
+  static void on_word_count_inventory_activate (GtkMenuItem *menuitem, gpointer user_data);
+  void on_word_count_inventory ();
+  static bool on_check_httpd_timeout (gpointer data);
+  void on_check_httpd ();
 
   /* Styles */
   static void on_insert_style_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_insert_style ();
+  void on_menu_insert_style ();
   static void on_file_styles (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_menu_file_styles ();
+  void on_menu_file_styles ();
   static void on_stylesheet_switch_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheet_switch ();
-  virtual void stylesheet_open_named (const ustring& stylesheet);
+  void on_stylesheet_switch ();
+  void stylesheet_open_named (const ustring& stylesheet);
   static void on_stylesheet_edit_mode_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheet_edit_mode ();
+  void on_stylesheet_edit_mode ();
   static void on_stylesheets_new_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheets_new ();
+  void on_stylesheets_new ();
   static void on_stylesheets_delete_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheets_delete ();
+  void on_stylesheets_delete ();
   static void on_stylesheets_rename_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheets_rename ();
+  void on_stylesheets_rename ();
   static void on_stylesheets_import_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheets_import ();
+  void on_stylesheets_import ();
   static void on_stylesheets_export_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheets_export ();
+  void on_stylesheets_export ();
   static void on_style_new_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_style_new ();
+  void on_style_new ();
   static void on_style_properties_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_style_delete ();
+  void on_style_delete ();
   static void on_style_delete_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_style_properties ();
+  void on_style_properties ();
   static gboolean on_treeview_styles_key_press_event (GtkWidget * widget, GdkEventKey * event, gpointer user_data);
   static gboolean on_treeview_styles_button_press_event (GtkWidget * widget, GdkEventButton * event, gpointer user_data);
-  virtual bool on_treeview_styles_button_press (GtkWidget * widget, GdkEventButton * event);
+  bool on_treeview_styles_button_press (GtkWidget * widget, GdkEventButton * event);
   static gboolean on_treeview_styles_popup_menu (GtkWidget *widget, gpointer user_data);
-  virtual void treeview_styles_popup_menu (GtkWidget *widget);
-  virtual void show_styles_popup_menu (GtkWidget *my_widget, GdkEventButton *event);
+  void treeview_styles_popup_menu (GtkWidget *widget);
+  void show_styles_popup_menu (GtkWidget *my_widget, GdkEventButton *event);
   static void on_treeview_styles_row_collapsed (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
   static void on_treeview_styles_row_expanded (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
-  virtual void on_treeview_styles_collapsed_expanded (GtkTreeIter *iter, GtkTreePath *path, bool expand);
+  void on_treeview_styles_collapsed_expanded (GtkTreeIter *iter, GtkTreePath *path, bool expand);
   static void on_stylesheets_expand_all_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheets_expand_all ();
+  void on_stylesheets_expand_all ();
   static void on_stylesheets_collapse_all_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_stylesheets_collapse_all ();
+  void on_stylesheets_collapse_all ();
   static void on_style_insert_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_style_insert ();
-  virtual bool on_style_expand ();
-  virtual bool on_style_collapse ();
+  void on_style_insert ();
+  bool on_style_expand ();
+  bool on_style_collapse ();
   
   /* Footnotes, endnotes, crossreferences */
   static void on_insert_footnote_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_insert_footnote ();
+  void on_insert_footnote ();
   static void on_insert_endnote_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_insert_endnote ();
+  void on_insert_endnote ();
   static void on_insert_crossreference_activate (GtkMenuItem *menuitem, gpointer user_data);
-  virtual void on_insert_crossreferencenote ();
+  void on_insert_crossreferencenote ();
 
   /* Formatted view */
   static gboolean on_text_motion_notify_event (GtkWidget *textview, GdkEventMotion *event, gpointer user_data);
-  virtual gboolean text_motion_notify_event (GtkWidget *textview, GdkEventMotion *event);
+  gboolean text_motion_notify_event (GtkWidget *textview, GdkEventMotion *event);
   static void on_text_event_after (GtkWidget *widget, GdkEvent *event, gpointer user_data);
-  virtual void text_event_after (GtkWidget *textview, GdkEvent *ev);
-  virtual void text_edit_if_link (GtkWidget *textview, GtkTextIter *iter);
+  void text_event_after (GtkWidget *textview, GdkEvent *ev);
+  void text_edit_if_link (GtkWidget *textview, GtkTextIter *iter);
   static gboolean text_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data);
-  virtual gboolean on_text_key_press_event (GtkWidget *widget, GdkEventKey *event);
+  gboolean on_text_key_press_event (GtkWidget *widget, GdkEventKey *event);
   static void on_textview_footnotes_move_cursor (GtkTextView * textview, GtkMovementStep step, gint count, gboolean extend_selection, gpointer user_data);
-  virtual void on_textview_footnotes_cursor_moved ();
+  void on_textview_footnotes_cursor_moved ();
   static void on_textview_footnotes_grab_focus (GtkWidget * widget, gpointer user_data);
-  virtual void on_textview_footnotes_focus_grabbed ();
+  void on_textview_footnotes_focus_grabbed ();
+
+  /* Synchronization */
+  static void on_synchronize_projects_activate (GtkMenuItem *menuitem, gpointer user_data);
+  void on_synchronize_projects ();
+  static void on_statusbutton_attention_clicked  (GtkButton *button, gpointer user_data);
+  void on_statusbutton_attention ();
+  void attention_show_hide (bool show);
+  bool attend_to_sync_project;
+  bool attend_to_sync_all;
+  static void on_preferences_synchronization_activate (GtkMenuItem *menuitem, gpointer user_data);
+  void on_preferences_synchronization ();
+  guint sync_source_id;
+  static bool synchronize_project_timeout (gpointer data);
+  void on_synchronize_project_timeout ();
+  void synchronize_project ();
+  
+  /* Tools */
+  static void on_menutools_activate (GtkMenuItem *menuitem, gpointer user_data);
+  static void on_line_cutter_for_hebrew_text1_activate (GtkMenuItem *menuitem, gpointer user_data);
+  void on_line_cutter_for_hebrew_text ();
+
 };
 
 
