@@ -23,13 +23,12 @@
 
 
 #include "libraries.h"
-#include "bookmetrics.h"
 
 
-void select_portion_get_values (BookMetrics& bookmetrics, const ustring& portion,
+void select_portion_get_values (const ustring& project, const ustring& book, const ustring& portion,
                                 unsigned int& chapter_from, ustring& verse_from,
                                 unsigned int& chapter_to, ustring& verse_to);
-ustring select_portion_get_label (BookMetrics& bookmetrics,
+ustring select_portion_get_label (const ustring& project, const ustring& book,
                                   unsigned int chapter_from, const ustring& verse_from,
                                   unsigned int chapter_to, const ustring& verse_to);
 
