@@ -24,11 +24,10 @@
 
 #include <gtk/gtk.h>
 #include "ustring.h"
-#include "configuration.h"
-#include "progress.h"
+#include "progresswindow.h"
 
 
-void view_project_pdf (Configuration * configuration, Progress * progress);
+void view_project_pdf (ProgressWindow& progresswindow);
 
 
 #endif
