@@ -24,14 +24,11 @@
 
 #include <gtk/gtk.h>
 #include "ustring.h"
-#include "configuration.h"
 #include "usfm.h"
-#include "progress.h"
-#include "session.h"
+#include "progresswindow.h"
 
 
-void view_references_pdf (Configuration * configuration, const vector<ustring>& references,
-                          Session * session, Progress * progress);
+void view_references_pdf (const vector<ustring>& references, ProgressWindow& progress);
 
 
 #endif
