@@ -23,14 +23,11 @@
 
 #include "libraries.h"
 #include <gtk/gtk.h>
-#include "configuration.h"
 
 
-ustring references_default_filename ();
-ustring references_searchwords_filename ();
-vector<ustring> references_hidden_ones_load (Configuration * configuration);
-void references_hidden_ones_save (vector <ustring>& references, Configuration * configuration);
-void references_hidden_ones_filter (vector <ustring>& references, vector <ustring>& comments, Configuration * configuration);
+vector<ustring> references_hidden_ones_load ();
+void references_hidden_ones_save (vector <ustring>& references);
+void references_hidden_ones_filter (vector <ustring>& references, vector <ustring>& comments);
 
 
 #endif
