@@ -21,20 +21,25 @@
 #ifndef INCLUDED_SCRIPTURE_REORDERED_H
 #define INCLUDED_SCRIPTURE_REORDERED_H
 
+
 #include "libraries.h"
-#include "scripture.h"
-#include "project.h"
 
 
 class ScriptureReordered
 {
 public:
-  ScriptureReordered (const Scripture& scripture, const Project& project);
-  ~ScriptureReordered ();
+  ScriptureReordered (const ustring& project);
   vector <ustring> books;
-  vector <ustring> paths;
   vector <ustring> portions;
+  vector<ustring> reordered_books;
+  vector<bool> reordered_includes;
+  vector<ustring> reordered_portions;
+  bool reordering_in_order ();
+  bool reordering_portions_all ();
 private:
+  void reordering_read ();
+  ustring myproject;
 };
 
+
 #endif
