@@ -26,42 +26,81 @@
 #include "gwrappers.h"
 #include "gtkwrappers.h"
 #include "scripturechecks.h"
-#include "scripture.h"
 #include "directories.h"
 #include "references.h"
-#include "project.h"
 #include "scripturechecksloadresults.h"
 #include "scripturechecksdisplayresultsbrowser.h"
 #include "dialogcheck.h"
 #include "style.h"
 #include "stylesheetutils.h"
 #include "shell.h"
+#include "projectutils.h"
+#include "generalconfig.h"
+#include "projectconfig.h"
+#include "progresswindow.h"
+#include "checks.h"
+#include "check_validate_usfm.h"
+#include "session.h"
+#include "check_count_words.h"
 
 
-ustring scripture_checks_internal_generate_filenames_combined (Configuration * configuration, Session * session)
+vector<ustring> scripture_checks_internal_generate_filenames_separate ()
 {
-  ustring filenames;
-  Scripture scripture (configuration->project);
-  for (unsigned int i = 0; i < scripture.paths.size(); i++) {
-    if (session->selected_books.find (scripture.books[i]) != session->selected_books.end())
-      filenames.append (shell_quote_space (scripture.paths[i]));
-  }    
+  // The filenames.
+  vector<ustring> filenames;
+  // Export all books.
+  ustring directory = gw_build_filename (directories_get_temp(), "checks");
+  create_directory (directory);
+  GeneralConfiguration genconfig (0);
+  ustring command;
+  command = "bibledit --export-project-usfm ";
+  command.append ("--project" + shell_quote_space (genconfig.project()));
+  command.append ("--directory" + shell_quote_space (directory));
+  command.append ("--gui");
+  system (command.c_str());
+  // Produce the filenames.
+  vector<ustring> books = project_get_books (genconfig.project());
+  Session session (0);
+  set<ustring> selected_books = session.selected_books ();
+  for (unsigned int i = 0; i < books.size(); i++) {
+    if (selected_books.find (books[i]) != selected_books.end())
+      filenames.push_back (shell_quote_space (gw_build_filename (directory, books[i] + ".usfm")));
+  } 
+  // Return result.
   return filenames;
 }
 
 
-vector<ustring> scripture_checks_internal_generate_filenames_separate (Configuration * configuration, Session * session)
+ustring scripture_checks_internal_generate_filenames_combined ()
 {
-  vector<ustring> filenames;
-  Scripture scripture (configuration->project);
-  for (unsigned int i = 0; i < scripture.paths.size(); i++) {
-    if (session->selected_books.find (scripture.books[i]) != session->selected_books.end())
-      filenames.push_back (shell_quote_space (scripture.paths[i]));
-  }    
+  // The filenames.
+  ustring filenames;
+  // Get the separate filenames.
+  vector <ustring> names = scripture_checks_internal_generate_filenames_separate ();
+  // Produce the filenames.
+  for (unsigned int i = 0; i < names.size(); i++) {
+    filenames.append (names[i]);
+  } 
+  // Return result.
   return filenames;
 }
 
 
+vector<ustring> checks_generate_booknames ()
+{
+  vector<ustring> booknames;
+  GeneralConfiguration genconfig (0);
+  Session session (0);
+  vector<ustring> books = project_get_books (genconfig.project());
+  set<ustring> selected_books = session.selected_books ();
+  for (unsigned int i = 0; i < books.size(); i++) {
+    if (selected_books.find (books[i]) != selected_books.end())
+      booknames.push_back (books[i]);
+  } 
+  return booknames;
+}
+
+
 ustring scripture_checks_internal_temporal_outputfile ()
 // Produces the filename for the messages in the temporal directory.
 {
@@ -83,7 +122,7 @@ void scripture_checks_internal_unlink_files ()
 }
 
 
-void scripture_checks_internal_display_results (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Configuration * configuration)
+void scripture_checks_internal_display_results (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 // Displays the results of the checks to the user.
 {
   // See if we have any error messages.
@@ -97,7 +136,7 @@ void scripture_checks_internal_display_results (GtkListStore * liststore, GtkWid
   }
   ScriptureChecksLoadResults sclr (scripture_checks_internal_temporal_outputfile ());
   References references (liststore, treeview, treecolumn);
-  references.set_references (sclr.references, sclr.comments, configuration);
+  references.set_references (sclr.references, sclr.comments);
   references.fill_store ();
 }
 
@@ -118,177 +157,141 @@ void scripture_checks_internal_display_results (CheckDialogType displaywhat)
 }
 
 
-void scripture_checks_chapters_verses (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
+void scripture_checks_chapters_verses (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 {
-  int dialogresponse;
   {
-    CheckDialog dialog (cdtChaptersVerses, configuration, session);
-    dialogresponse = dialog.run();
+    CheckDialog dialog (cdtChaptersVerses);
+    if (dialog.run() != GTK_RESPONSE_OK)
+      return;
   }
-  if (dialogresponse != GTK_RESPONSE_OK)
-    return;
-  progress->start();
+  ProgressWindow progresswindow ("", true);
   scripture_checks_internal_unlink_files ();
   vector<ustring> filenames;
-  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
-  progress->set_iterate (0, 1, filenames.size());
-  Project project (configuration->project);
+  filenames = scripture_checks_internal_generate_filenames_separate ();
+  progresswindow.set_iterate (0, 1, filenames.size());
+  ProjectConfiguration projectconfig ("");
   for (unsigned int i = 0; i < filenames.size(); i++) {
-    progress->iterate ();
-    if (progress->cancel)
+    progresswindow.iterate ();
+    if (progresswindow.cancel)
       break;
     ustring command;
     command = "sc-input-usfm " + filenames[i]; // Do not quote here - done before.
-    command.append (" | sc-chapters-verses --versification " + project.versification);
+    command.append (" | sc-chapters-verses --versification " + projectconfig.versification());
     command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
     command.append ("2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile()));
     system (command.c_str());
   }
-  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
-  progress->finish();
+  scripture_checks_internal_display_results (liststore, treeview, treecolumn);
 }
 
 
-void scripture_checks_count_usfms (Configuration * configuration, Session * session, Progress * progress)
+void scripture_checks_count_usfms ()
 {
   int dialogresponse;
   {
-    CheckDialog dialog (cdtMarkersCount, configuration, session);
+    CheckDialog dialog (cdtMarkersCount);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
-  progress->start();
+  ProgressWindow progresswindow ("", true);
   scripture_checks_internal_unlink_files ();
   ustring filenames;
-  filenames = scripture_checks_internal_generate_filenames_combined (configuration, session);
-  progress->set_fraction (0.5);
+  filenames = scripture_checks_internal_generate_filenames_combined ();
+  progresswindow.set_fraction (0.5);
+  Session session (0);
   ustring command;
   command = "sc-count-usfm ";
-  if (session->checkmarkercounttype == cmctSortUsfm)
+  if (session.checksorttype() == cstSort1)
     command.append ("--sort-on-usfm");
-  if (session->checkmarkercounttype == cmctSortCount)
+  if (session.checksorttype() == cstSort2)
     command.append ("--sort-on-count");
   command.append (" " + filenames); // Do not quote here - done before.
   command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
   command.append ("2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile()));
   system (command.c_str());
-  progress->set_fraction (1);
+  progresswindow.set_fraction (1);
   scripture_checks_internal_display_results (cdtMarkersCount);
-  progress->finish();
 }
 
 
-void scripture_checks_validate_usfms (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
+void scripture_checks_validate_usfms (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 {
-  // Handle dialog for the user.
   int dialogresponse;
   {
-    CheckDialog dialog (cdtMarkersValidate, configuration, session);
+    CheckDialog dialog (cdtMarkersValidate);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
-  // Progress information.
-  progress->start();
-  // Clear up old stuff.
-  scripture_checks_internal_unlink_files ();
-  // Get all filenames to check for.
-  vector<ustring> filenames;
-  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
-  // Get all markers we have in the stylesheet.
-  ustring markers;
-  {
-    Project project (configuration);
-    vector<Style> styles;
-    stylesheet_get_styles (project.get_stylesheet(), styles);
-    for (unsigned int i = 0; i < styles.size(); i++) {
-      if (!markers.empty())
-        markers.append (" ");
-      markers.append (styles[i].marker);
-      if (styles[i].type == stInlineText) {
-        markers.append (" " + styles[i].marker + "*");
-      }
-    }
-  }
-  // Progress.
-  progress->set_iterate (0, 1, filenames.size());
-  // Go through each filename and check it.
-  for (unsigned int i = 0; i < filenames.size(); i++) {
-    // Progress / cancel information.
-    progress->iterate ();
-    if (progress->cancel)
-      break;
-    // Run the program for the library, that does the actual checking.
-    ustring command;
-    command = "sc-input-usfm " + filenames[i] // Do not quote here - done before.
-      + " | sc-check-usfm --markers-in-stylesheet '" + markers + "'"
-      + " >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile())
-      + "2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile());
-    system (command.c_str());
-  }
-  // Display results, and finish progress.
-  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
-  progress->finish();
+  GeneralConfiguration genconfig (0);
+  CheckValidateUsfm check (genconfig.project(), checks_generate_booknames (), true, true);
+  checks_display_references_comments (check.references, check.comments, liststore, treeview, treecolumn);
 }
 
 
-void scripture_checks_compare_usfms (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
+void scripture_checks_compare_usfms (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 {
+  // Configuration.
+  GeneralConfiguration genconfig (0);
+  Session session (0);
   // Open dialog to present information to user and to get the options from him.
   int dialogresponse;
   {
-    CheckDialog dialog (cdtMarkersCompare, configuration, session);
+    CheckDialog dialog (cdtMarkersCompare);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
-  // Open the two scripture.
-  Scripture scripture (configuration->project);
-  Scripture scripture2 (configuration->check_markers_compare_project);
+  // Get the books of the two projects.
+  vector<ustring> scripture_books = project_get_books (genconfig.project());
+  vector<ustring> scripture2_books = project_get_books (genconfig.check_markers_compare_project());
   // Container with flags to verify all books in scripture2 have been dealt with.
   vector<bool> scripture2_checked;
-  for (unsigned int i = 0; i < scripture2.books.size(); i++)
+  for (unsigned int i = 0; i < scripture2_books.size(); i++)
     scripture2_checked.push_back (false);
   // Display progress.
-  progress->start();
+  ProgressWindow progresswindow ("", true);
   scripture_checks_internal_unlink_files ();
-  progress->set_iterate (0, 1, scripture.paths.size());
+  progresswindow.set_iterate (0, 1, scripture_books.size());
   // Go through each book we have in this scripture.
-  for (unsigned int i = 0; i < scripture.paths.size(); i++) {
-    progress->iterate ();
-    if (progress->cancel)
+  set <ustring> selected_books = session.selected_books ();
+  for (unsigned int i = 0; i < scripture_books.size(); i++) {
+    progresswindow.iterate ();
+    if (progresswindow.cancel)
       break;
     // See whether this book is included in the ones to check.
-    if (session->selected_books.find (scripture.books[i]) != session->selected_books.end()) {
-      // Get a pointer to the same book in the second scripture.
+    if (selected_books.find (scripture_books[i]) != selected_books.end()) {
+      // Get pointer to same book in second scripture.
       int pointer2 = -1;
-      for (unsigned int i2 = 0; i2 < scripture2.books.size(); i2++) {
-        if (scripture.books[i] == scripture2.books[i2]) {
+      for (unsigned int i2 = 0; i2 < scripture2_books.size(); i2++) {
+        if (scripture_books[i] == scripture2_books[i2]) {
           pointer2 = i2;
           break;
         }
       }
+      // Command to execute.
       ustring command;
-      if (pointer2 >= 0) {
+      // Does the book exist in the second project?
+      if (project_book_exists (genconfig.check_markers_compare_project(), scripture_books[i])) {
         // Ok, same book found in second scripture.
-        ustring file1 = shell_quote_space (scripture.paths[i]);
-        ustring file2 = shell_quote_space (scripture2.paths[pointer2]);
+        ustring file1 = shell_quote_space (scripture_books[i]);
+        ustring file2 = shell_quote_space (scripture2_books[pointer2]);
         command = "sc-compare-usfm" + shell_quote_space (file1) + shell_quote_space (file2);
-        if ((!configuration->check_markers_compare_all_markers) && (!configuration->check_markers_compare_include_only.empty())) {
-          command.append (" --include-only '" + configuration->check_markers_compare_include_only + "'");
+        if ((!genconfig.check_markers_compare_all_markers()) && (!genconfig.check_markers_compare_include_only().empty())) {
+          command.append (" --include-only '" + genconfig.check_markers_compare_include_only() + "'");
         }
-        if (!configuration->check_markers_compare_ignore.empty()) {
-          command.append (" --ignore '" + configuration->check_markers_compare_ignore + "'");
+        if (!genconfig.check_markers_compare_ignore().empty()) {
+          command.append (" --ignore '" + genconfig.check_markers_compare_ignore() + "'");
         }
-        if (configuration->check_markers_compare_ignore_verse_zero)
+        if (genconfig.check_markers_compare_ignore_verse_zero())
           command.append (" --ignore-verse-zero");
         // Mark this book as done.
         scripture2_checked [pointer2] = true;
       }
       else {
         // Same book not found: give message about that.
-        command = "sc-message '" + scripture.books[i]
+        command = "sc-message '" + scripture_books[i]
           + "' 0 0 'Extra book'";
       }
       command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
@@ -299,11 +302,11 @@ void scripture_checks_compare_usfms (Configuration * configuration, Session * se
   // See whether there are still some unchecked books in the second scripture.
   // Give message if there are unchecked ones, but only in case ALL books
   // were selected for checking.
-  if (scripture.books.size() == session->selected_books.size()) {
+  if (scripture_books.size() == session.selected_books().size()) {
     for (unsigned int i = 0; i < scripture2_checked.size(); i++) {
       if (!scripture2_checked[i]) {
         ustring command;
-        command = "sc-message '" + scripture2.books[i]
+        command = "sc-message '" + scripture2_books[i]
           + "' 0 0 'Missing book'";
         command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
         command.append ("2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile()));
@@ -312,211 +315,236 @@ void scripture_checks_compare_usfms (Configuration * configuration, Session * se
     }
   }
   // Display results and finish.
-  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
-  progress->finish();
+  scripture_checks_internal_display_results (liststore, treeview, treecolumn);
 }
 
 
-void scripture_checks_count_characters (Configuration * configuration, Session * session, Progress * progress)
+void scripture_checks_count_characters ()
 {
   int dialogresponse;
   {
-    CheckDialog dialog (cdtCharactersCount, configuration, session);
+    CheckDialog dialog (cdtCharactersCount);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
   scripture_checks_internal_unlink_files ();
   ustring filenames;
-  filenames = scripture_checks_internal_generate_filenames_combined (configuration, session);
-  progress->set_fraction (0.5);
+  filenames = scripture_checks_internal_generate_filenames_combined ();
+  ProgressWindow progresswindow ("", false);
+  progresswindow.set_fraction (0.5);
+  Session session (0);
   ustring command;
   command = "sc-input-usfm --categorize ";
   command.append (filenames); // Do not quote here - done before.
   command.append (" | sc-count-characters ");
-  if (session->checkcharacterscounttype == chctNoSort)
+  if (session.checksorttype() == cstSort0)
     command.append ("--no-sort");
-  if (session->checkcharacterscounttype == chctSortUsfm)
+  if (session.checksorttype() == cstSort1)
     command.append ("--sort-on-character");
-  if (session->checkcharacterscounttype == chctSortCount)
+  if (session.checksorttype() == cstSort2)
     command.append ("--sort-on-count");
   command.append (" --invisible-characters");
   command.append (" >> " + scripture_checks_internal_temporal_outputfile());
   command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
-  run_shell_progress (command, "sc-count-characters", progress, "Counting", false, 0);
+  run_shell_progress (command, "sc-count-characters", true, "Counting", false, 0);
   scripture_checks_internal_display_results (cdtCharactersCount);
 }
 
 
-void scripture_checks_unwanted_patterns (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
+void scripture_checks_unwanted_patterns (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 {
+  GeneralConfiguration genconfig (0);
   int dialogresponse;
   {
-    CheckDialog dialog (cdtUnwantedPatterns, configuration, session);
+    CheckDialog dialog (cdtUnwantedPatterns);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
-  progress->start();
+  ProgressWindow progresswindow ("", true);
   scripture_checks_internal_unlink_files ();
   vector<ustring> filenames;
-  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
-  progress->set_iterate (0, 1, filenames.size());
+  filenames = scripture_checks_internal_generate_filenames_separate ();
+  progresswindow.set_iterate (0, 1, filenames.size());
   for (unsigned int i = 0; i < filenames.size(); i++) {
-    progress->iterate ();
-    if (progress->cancel)
+    progresswindow.iterate ();
+    if (progresswindow.cancel)
       break;
     ustring command;
     command = "sc-input-usfm --categorize " + filenames[i]; // Do not quote here - done before.
-    command.append (" | sc-unwanted-patterns '" + checks_unwanted_patterns_get_filename (configuration->project) + "'");
+    command.append (" | sc-unwanted-patterns '" + checks_unwanted_patterns_get_filename (genconfig.project()) + "'");
     command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
     command.append ("2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile()));
     system (command.c_str());
   }
-  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
-  progress->finish();
+  scripture_checks_internal_display_results (liststore, treeview, treecolumn);
 }
 
 
-void scripture_checks_capitalization (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
+void scripture_checks_capitalization (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 {
+  GeneralConfiguration genconfig (0);
   int dialogresponse;
   {
-    CheckDialog dialog (cdtCapitalization, configuration, session);
+    CheckDialog dialog (cdtWordsCapitalization);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
-  progress->start();
+  ProgressWindow progresswindow ("", true);
   scripture_checks_internal_unlink_files ();
   vector<ustring> filenames;
-  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
-  progress->set_iterate (0, 1, filenames.size());
+  filenames = scripture_checks_internal_generate_filenames_separate ();
+  progresswindow.set_iterate (0, 1, filenames.size());
   for (unsigned int i = 0; i < filenames.size(); i++) {
-    progress->iterate ();
-    if (progress->cancel)
+    progresswindow.iterate ();
+    if (progresswindow.cancel)
       break;
     ustring command;
     command = "sc-input-usfm --categorize " + filenames[i]; // Do not quote here - done before.
     command.append (" | sc-capitalization");
-    if (!configuration->check_capitalization_punctuation.empty())
-      command.append (" --punctuation-followed-by-capitals '" + configuration->check_capitalization_punctuation + "'");
-    if (!configuration->check_capitalization_ignore.empty())
-      command.append (" --ignore-lowercase-following '" + configuration->check_capitalization_ignore + "'");
-    command.append (" --abbreviations '" + checks_abbreviations_get_filename (configuration->project) + "'");    
-    if (configuration->check_capitalization_allow_any_prefixes)
+    if (!genconfig.check_capitalization_punctuation().empty())
+      command.append (" --punctuation-followed-by-capitals '" + genconfig.check_capitalization_punctuation() + "'");
+    if (!genconfig.check_capitalization_ignore().empty())
+      command.append (" --ignore-lowercase-following '" + genconfig.check_capitalization_ignore() + "'");
+    command.append (" --abbreviations '" + checks_abbreviations_get_filename (genconfig.project()) + "'");    
+    if (genconfig.check_capitalization_allow_any_prefixes())
       command.append (" --any-prefixes");
     else
-      command.append (" --uncapitalized-prefixes '" + checks_uncapitalized_prefixes_get_filename (configuration->project) + "'");    
-    command.append (" --capitalized-suffixes '" + checks_capitalized_suffixes_get_filename (configuration->project) + "'");    
+      command.append (" --uncapitalized-prefixes '" + checks_uncapitalized_prefixes_get_filename (genconfig.project()) + "'");    
+    command.append (" --capitalized-suffixes '" + checks_capitalized_suffixes_get_filename (genconfig.project()) + "'");    
     command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
     command.append ("2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile()));
     system (command.c_str());
   }
-  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
-  progress->finish();
+  scripture_checks_internal_display_results (liststore, treeview, treecolumn);
 }
 
 
-void scripture_checks_repetition (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
+void scripture_checks_repetition (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 {
+  GeneralConfiguration genconfig (0);
   int dialogresponse;
   {
-    CheckDialog dialog (cdtRepetition, configuration, session);
+    CheckDialog dialog (cdtWordsRepetition);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
-  progress->start();
+  ProgressWindow progresswindow ("", true);
   scripture_checks_internal_unlink_files ();
   vector<ustring> filenames;
-  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
-  progress->set_iterate (0, 1, filenames.size());
+  filenames = scripture_checks_internal_generate_filenames_separate ();
+  progresswindow.set_iterate (0, 1, filenames.size());
   for (unsigned int i = 0; i < filenames.size(); i++) {
-    progress->iterate ();
-    if (progress->cancel)
+    progresswindow.iterate ();
+    if (progresswindow.cancel)
       break;
     ustring command;
     command = "sc-input-usfm --categorize " + filenames[i]; // Do not quote here - done before.
     command.append (" | sc-repetition");
-    if (configuration->check_repetition_ignore_case)
+    if (genconfig.check_repetition_ignore_case())
       command.append (" --ignore-case");
-    if (configuration->check_repetition_show_only_these)
-      command.append (" --show-only '" + checks_repetition_show_only_get_filename (configuration->project) + "'");
-    if (configuration->check_repetition_ignore_these)
-      command.append (" --ignore '" + checks_repetition_ignore_get_filename (configuration->project) + "'");
+    if (genconfig.check_repetition_show_only_these())
+      command.append (" --show-only '" + checks_repetition_show_only_get_filename (genconfig.project()) + "'");
+    if (genconfig.check_repetition_ignore_these())
+      command.append (" --ignore '" + checks_repetition_ignore_get_filename (genconfig.project()) + "'");
     command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
     command.append ("2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile()));
     system (command.c_str());
   }
-  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
-  progress->finish();
+  scripture_checks_internal_display_results (liststore, treeview, treecolumn);
 }
 
 
-void scripture_checks_matching_pairs (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
+void scripture_checks_matching_pairs (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 {
+  GeneralConfiguration genconfig (0);
   int dialogresponse;
   {
-    CheckDialog dialog (cdtMatchingPairs, configuration, session);
+    CheckDialog dialog (cdtMatchingPairs);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
-  progress->start();
+  ProgressWindow progresswindow ("", true);
   scripture_checks_internal_unlink_files ();
   vector<ustring> filenames;
-  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
-  progress->set_iterate (0, 1, filenames.size());
+  filenames = scripture_checks_internal_generate_filenames_separate ();
+  progresswindow.set_iterate (0, 1, filenames.size());
   for (unsigned int i = 0; i < filenames.size(); i++) {
-    progress->iterate ();
-    if (progress->cancel)
+    progresswindow.iterate ();
+    if (progresswindow.cancel)
       break;
     ustring command;
     command = "sc-input-usfm --categorize " + filenames[i]; // Do not quote here - done before.
     command.append (" | sc-matching-pairs");
-    if (!configuration->check_matching_pairs_ignore.empty()) {
+    if (!genconfig.check_matching_pairs_ignore().empty()) {
       command.append (" --ignore '");
-      command.append (configuration->check_matching_pairs_ignore);
+      command.append (genconfig.check_matching_pairs_ignore());
       command.append ("'");
     }
     command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
     command.append ("2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile()));
     system (command.c_str());
   }
-  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
-  progress->finish();
+  scripture_checks_internal_display_results (liststore, treeview, treecolumn);
 }
 
 
-void scripture_checks_unwanted_words (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
+void scripture_checks_unwanted_words (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
 {
+  GeneralConfiguration genconfig (0);
   int dialogresponse;
   {
-    CheckDialog dialog (cdtUnwantedWords, configuration, session);
+    CheckDialog dialog (cdtWordsUnwanted);
     dialogresponse = dialog.run();
   }
   if (dialogresponse != GTK_RESPONSE_OK)
     return;
-  progress->start();
+  ProgressWindow progresswindow ("", true);
   scripture_checks_internal_unlink_files ();
   vector<ustring> filenames;
-  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
-  progress->set_iterate (0, 1, filenames.size());
+  filenames = scripture_checks_internal_generate_filenames_separate ();
+  progresswindow.set_iterate (0, 1, filenames.size());
   for (unsigned int i = 0; i < filenames.size(); i++) {
-    progress->iterate ();
-    if (progress->cancel)
+    progresswindow.iterate ();
+    if (progresswindow.cancel)
       break;
     ustring command;
     command = "sc-input-usfm --categorize " + filenames[i]; // Do not quote here - done before.
     command.append (" | sc-unwanted-words");
-    command.append (" '" + checks_unwanted_words_get_filename (configuration->project) + "'");
+    command.append (" '" + checks_unwanted_words_get_filename (genconfig.project()) + "'");
     command.append (" >>" + shell_quote_space (scripture_checks_internal_temporal_outputfile()));
     command.append ("2>>" + shell_quote_space (scripture_checks_internal_temporal_errorfile()));
     system (command.c_str());
   }
-  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
-  progress->finish();
+  scripture_checks_internal_display_results (liststore, treeview, treecolumn);
+}
+
+
+void scripture_checks_word_inventory (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn)
+{
+  int dialogresponse;
+  {
+    CheckDialog dialog (cdtWordsCount);
+    dialogresponse = dialog.run();
+  }
+  if (dialogresponse != GTK_RESPONSE_OK)
+    return;
+  GeneralConfiguration genconfig (0);
+  Session session (0);
+  CheckCountWords check (genconfig.project(), checks_generate_booknames (), 
+                         genconfig.check_words_inventory_word_forming_characters(),
+                         session.checksorttype () == cstSort1, 
+                         session.checksorttype () == cstSort2, 
+                         genconfig.check_words_inventory_not_include_words_count (), 
+                         true);
+  DisplayCheckingResults display ("Word Inventory");
+  display.word_inventory (check.words, check.counts, 
+                          check.total_count, check.total_unique_count, check.filtered_count, check.filtered_unique_count, 
+                          genconfig.check_words_inventory_not_include_words_count ());
 }
 
 
