@@ -17,21 +17,18 @@
 **  
 */
 
+
 #ifndef INCLUDED_SEARCH_UTILS_H
 #define INCLUDED_SEARCH_UTILS_H
 
 
 #include "libraries.h"
-#include "progress.h"
-#include "session.h"
 #include "bibletime.h"
 
 
-void search_string (Session * session, const ustring& project, 
-                    bool use_books_selection, vector<ustring>& results);
-void text_was_being_indexed_warning (GtkWidget * parentwindow);
-void search_string (Configuration * configuration, GtkListStore * liststore, 
-                    GtkWidget * listview, GtkTreeViewColumn * treeviewcolumn, 
-                    Progress * progress, Session * session, BibleTime * bibletime);
+void search_string_basic (const ustring& project, bool use_book_selection, unsigned int chapter, vector<ustring>& results);
+void search_string (GtkListStore * liststore, GtkWidget * listview, GtkTreeViewColumn * treeviewcolumn, 
+                    BibleTime * bibletime);
+
 
 #endif
