@@ -17,6 +17,7 @@
 **  
 */
 
+
 #ifndef INCLUDED_SHELL_H
 #define INCLUDED_SHELL_H
 
@@ -24,14 +25,17 @@
 #include "libraries.h"
 #include <glib.h>
 #include <gtk/gtk.h>
-#include "progress.h"
 
 
 bool program_is_running (const ustring & commandline);
+int programs_running_count (const ustring & commandline);
+bool program_is_running_basic (const ustring & program);
+int programs_running_count_basic (const ustring & program);
 int run_shell_progress (const ustring& commandline, const ustring& programname, 
-                        Progress * progress, const ustring& message, 
+                        bool gui, const ustring& message, 
                         bool output_to_logfile, int max_logfile_size);
 ustring shell_quote_space (const ustring& filename);
+bool copy_file (const ustring& from, const ustring& to);
 
 
 #endif
