@@ -26,14 +26,15 @@
 #include "usfmtools.h"
 #include <map>
 #include "dialogentry.h"
-#include "projects.h"
+#include "projectutils.h"
 #include "directories.h"
-#include "project.h"
 #include "utilities.h"
 #include "shell.h"
+#include "generalconfig.h"
+#include "projectconfig.h"
 
 
-void styletree_load_stylesheet (GtkTreeStore * store, const ustring& stylesheet, GtkTreeView * treeview, Configuration * configuration)
+void styletree_load_stylesheet (GtkTreeStore * store, const ustring& stylesheet, GtkTreeView * treeview)
 // Load a whole stylesheet.
 {
   vector<ustring> markers;
@@ -65,9 +66,8 @@ void styletree_load_stylesheet (GtkTreeStore * store, const ustring& stylesheet,
     row = markers[i] + " " + names[markers[i]];
     gtk_tree_store_set (store, &child_iter, 0, row.c_str(), -1);
     if (insert_category) {
-      bool * state;
-      state = styletree_get_pointer_to_state (ucategory, configuration);
-      if (* state) {
+      bool state = styletree_get_state (ucategory);
+      if (state) {
         GtkTreePath * path;
         path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
         gtk_tree_view_expand_row (treeview, path, false);
@@ -83,15 +83,17 @@ void styletree_load_stylesheet (GtkTreeStore * store, const ustring& stylesheet,
 #define RECENTLY_USED "Recently Used"
 
 
-void styletree_load_recently_used_styles (GtkTreeStore * store, GtkTreeView * treeview, Configuration * configuration)
+void styletree_load_recently_used_styles (GtkTreeStore * store, GtkTreeView * treeview)
 // Sets the recently used styles in the treeview.
 {
+  // Configuration
+  GeneralConfiguration genconfig (0);
   // Get the often used styles, if there are any.
-  vector<ustring> styles = stylesheet_get_recently_used (configuration->stylesheet);
+  vector<ustring> styles = stylesheet_get_recently_used (genconfig.stylesheet());
   {
     // Make it a complete style.
     for (unsigned int i = 0; i < styles.size(); i++) {
-      Style styleobject (configuration->stylesheet, styles[i], false);
+      Style styleobject (genconfig.stylesheet(), styles[i], false);
       styles[i] = styles[i] + " " + styleobject.name;
     }
   }
@@ -108,9 +110,8 @@ void styletree_load_recently_used_styles (GtkTreeStore * store, GtkTreeView * tr
       gtk_tree_store_set (store, &child_iter, 0, styles[i].c_str(), -1);
     }
     // Expand if needed.
-    bool * state;
-    state = styletree_get_pointer_to_state (RECENTLY_USED, configuration);
-    if (* state) {
+    bool state = styletree_get_state (RECENTLY_USED);
+    if (state) {
       GtkTreePath * path;
       path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
       gtk_tree_view_expand_row (treeview, path, false);
@@ -123,10 +124,12 @@ void styletree_load_recently_used_styles (GtkTreeStore * store, GtkTreeView * tr
 }
 
 
-void styletree_clear_recently_used_style (Configuration * configuration, const ustring& style)
+void styletree_clear_recently_used_style (const ustring& style)
 {
+  // Configuration
+  GeneralConfiguration genconfig (0);
   // Get the styles we now have.
-  vector<ustring> styles1 = stylesheet_get_recently_used (configuration->stylesheet);
+  vector<ustring> styles1 = stylesheet_get_recently_used (genconfig.stylesheet());
   // Remove the style.
   vector<ustring> styles2;
   for (unsigned int i = 0; i < styles1.size(); i++) {
@@ -134,32 +137,35 @@ void styletree_clear_recently_used_style (Configuration * configuration, const u
       styles2.push_back(styles1[i]);
   }
   // Store the styles again.
-  stylesheet_set_recently_used (configuration->stylesheet, styles2);
+  stylesheet_set_recently_used (genconfig.stylesheet(), styles2);
 }
 
 
-void styletree_clear_recently_used_styles (Configuration * configuration)
+void styletree_clear_recently_used_styles ()
 {
+  GeneralConfiguration genconfig (0);
   vector<ustring> styles;
-  stylesheet_set_recently_used (configuration->stylesheet, styles);
+  stylesheet_set_recently_used (genconfig.stylesheet(), styles);
 }
 
 
-void styletree_use_style (GtkTreeStore * store, GtkWidget * treeview, Configuration * configuration, const ustring& style)
+void styletree_use_style (GtkTreeStore * store, GtkWidget * treeview, const ustring& style)
 // "Uses" a style, and so ranks it at the top of the recently used styles,
 // and updates both the store and the configuration.
 {
+  // Configuration
+  GeneralConfiguration genconfig (0);
   // Make a "complete" style, that is, style and name.
   ustring complete_style;
   {
-    Style styleobject (configuration->stylesheet, style, false);
+    Style styleobject (genconfig.stylesheet(), style, false);
     complete_style = style + " " + styleobject.name;
   }
   // Get the styles we have often used.
-  vector<ustring> old_styles = stylesheet_get_recently_used (configuration->stylesheet);
+  vector<ustring> old_styles = stylesheet_get_recently_used (genconfig.stylesheet());
   // Make them complete styles.
   for (unsigned int i = 0; i < old_styles.size(); i++) {
-    Style styleobject (configuration->stylesheet, old_styles[i], false);
+    Style styleobject (genconfig.stylesheet(), old_styles[i], false);
     old_styles[i] = old_styles[i] + " " + styleobject.name;
   }
   // Obtain a pointer to the style we have often used.
@@ -177,6 +183,9 @@ void styletree_use_style (GtkTreeStore * store, GtkWidget * treeview, Configurat
       new_styles.push_back (old_styles[i]);
     }
   }
+  // Display no more than a maximum number of styles.
+  while (new_styles.size() > 30)
+    new_styles.erase (new_styles.end());
   // Display information.
   GtkTreeModel * model;
   model = GTK_TREE_MODEL (store);
@@ -200,9 +209,8 @@ void styletree_use_style (GtkTreeStore * store, GtkWidget * treeview, Configurat
     gtk_tree_store_set (store, &child_iter, 0, new_styles[i].c_str(), -1);
   }
   // Expand if needed.
-  bool * state;
-  state = styletree_get_pointer_to_state (RECENTLY_USED, configuration);
-  if (* state) {
+  bool state = styletree_get_state (RECENTLY_USED);
+  if (state) {
     GtkTreePath * path;
     path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
     gtk_tree_view_expand_row (GTK_TREE_VIEW (treeview), path, false);
@@ -213,60 +221,95 @@ void styletree_use_style (GtkTreeStore * store, GtkWidget * treeview, Configurat
     Parse parse (new_styles[i]);
     new_styles[i] = parse.words[0];
   }
-  stylesheet_set_recently_used (configuration->stylesheet, new_styles);
+  stylesheet_set_recently_used (genconfig.stylesheet(), new_styles);
 }
 
 
-void styletree_store_state (GtkTreeStore * store, Configuration * configuration, GtkTreeIter *iter, bool expand)
+size_t styletree_get_state_offset (const ustring& row) 
+{
+  size_t offset = 0; // Default: recently used.
+  if (row == usfm_get_category_name (ucIdentificationInformation))
+    offset = 1;
+  if (row == usfm_get_category_name (ucIntroductionTitlesHeadings))
+    offset = 2;
+  if (row == usfm_get_category_name (ucIntroductionParagraphsPoetry))
+    offset = 3;
+  if (row == usfm_get_category_name (ucIntroductionOtherElements))
+    offset = 4;
+  if (row == usfm_get_category_name (ucTitles))
+    offset = 5;
+  if (row == usfm_get_category_name (ucHeadings))
+    offset = 6;
+  if (row == usfm_get_category_name (ucChaptersAndVerses))
+    offset = 7;
+  if (row == usfm_get_category_name (ucParagraphs))
+    offset = 8;
+  if (row == usfm_get_category_name (ucLists))
+    offset = 9;
+  if (row == usfm_get_category_name (ucPoetryElements))
+    offset = 10;
+  if (row == usfm_get_category_name (ucTableElements))
+    offset = 11;
+  if (row == usfm_get_category_name (ucFootnotes))
+    offset = 12;
+  if (row == usfm_get_category_name (ucCrossReferences))
+    offset = 13;
+  if (row == usfm_get_category_name (ucExtendedStudyNotes))
+    offset = 14;
+  if (row == usfm_get_category_name (ucSpecialText))
+    offset = 15;
+  if (row == usfm_get_category_name (ucCharacterStyles))
+    offset = 16;
+  if (row == usfm_get_category_name (ucSpacingsAndBreaks))
+    offset = 17;
+  if (row == usfm_get_category_name (ucSpecialFeatures))
+    offset = 18;
+  if (row == usfm_get_category_name (ucPeripheralMaterials))
+    offset = 19;
+  if (row == usfm_get_category_name (ucNonstandardStyles))
+    offset = 20;
+  return offset;
+}
+
+
+void styletree_store_state (GtkTreeStore * store, GtkTreeIter *iter, bool expand)
 {
   // Get the name of the row that has been affected.
   gchar *str_data;
   gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 0, &str_data, -1);
   ustring row = str_data;
   g_free (str_data);
-  // Store it in the configuration.
-  bool * state;
-  state = styletree_get_pointer_to_state (row, configuration);
-  * state = expand;
+  // Store it in the configuration. Cater for 20 states.
+  GeneralConfiguration genconfig (0);
+  vector<bool> states = genconfig.styles_category_expanded ();
+  if (states.size() < 20) {
+    unsigned int add = 20 - states.size();
+    for (unsigned int i = 0; i < add; i++)
+      states.push_back (false);
+  }
+  size_t offset = styletree_get_state_offset (row);
+  try {
+    states[offset] = expand;
+    genconfig.styles_category_expanded_set (states);
+  }
+  catch (exception & ex)
+  {
+    gw_critical (ex.what ());
+  }
 }
 
 
-bool * styletree_get_pointer_to_state (const ustring& row, Configuration * configuration)
-// Taking the string in "row", this returns a pointer to the boolean state in configuration.
+bool styletree_get_state (const ustring& row)
+// Taking the string in "row", this returns the state as in configuration.
 {
-  // Look for the categorised rows, and if found return pointer to the state.
-  if (row == usfm_get_category_name (ucIdentificationInformation))
-    return &(configuration->category_expanded_1);
-  if (row == usfm_get_category_name (ucIntroductionTitlesHeadings))
-    return &(configuration->category_expanded_2a);
-  if (row == usfm_get_category_name (ucIntroductionParagraphsPoetry))
-    return &(configuration->category_expanded_2b);
-  if (row == usfm_get_category_name (ucIntroductionOtherElements))
-    return &(configuration->category_expanded_2c);  
-  if (row == usfm_get_category_name (ucTitlesHeadings))
-    return &(configuration->category_expanded_3);
-  if (row == usfm_get_category_name (ucChaptersAndVerses))
-    return &(configuration->category_expanded_4);
-  if (row == usfm_get_category_name (ucParagraphElements))
-    return &(configuration->category_expanded_5);
-  if (row == usfm_get_category_name (ucPoetryElements))
-    return &(configuration->category_expanded_6);
-  if (row == usfm_get_category_name (ucTableElements))
-    return &(configuration->category_expanded_7);
-  if (row == usfm_get_category_name (ucFootnotes))
-    return &(configuration->category_expanded_8);
-  if (row == usfm_get_category_name (ucCrossReferences))
-    return &(configuration->category_expanded_9);
-  if (row == usfm_get_category_name (ucExtendedStudyNotes))
-    return &(configuration->category_expanded_10);
-  if (row == usfm_get_category_name (ucSpecialTextAndCharacterStyles))
-    return &(configuration->category_expanded_11);
-  if (row == usfm_get_category_name (ucPeripheralMaterials))
-    return &(configuration->category_expanded_12);
-  if (row == usfm_get_category_name (ucNonstandardStyles))
-    return &(configuration->category_expanded_13);
-  // Default: Recently used category.
-  return &(configuration->category_expanded_0);
+  // Look for the categorised rows, and if found return the state.
+  bool state = false;
+  GeneralConfiguration genconfig (0);
+  vector<bool> states = genconfig.styles_category_expanded();
+  size_t offset = styletree_get_state_offset (row);
+  if (offset < states.size())
+    state = states[offset];
+  return state;
 }
 
 
@@ -533,10 +576,11 @@ ustring styletree_get_string_before_focus (GtkWidget * treeview)
 }
 
 
-void styletree_rename_stylesheet (Configuration * configuration, GtkWidget * parent)
+void styletree_rename_stylesheet (GtkWidget * parent)
 {
   // Get the current name of the stylesheet, and ask for a new name.
-  ustring currentname = configuration->stylesheet;
+  GeneralConfiguration genconfig (0);
+  ustring currentname = genconfig.stylesheet();
   EntryDialog dialog ("Rename stylesheet", "Give a new name for this stylesheet", currentname);
   if (dialog.run() == GTK_RESPONSE_OK) {
     // Check whether the new name does not already exist.
@@ -544,17 +588,17 @@ void styletree_rename_stylesheet (Configuration * configuration, GtkWidget * par
       // Ok, new name is still free.
       // Check all projects and if they have this stylesheet, rename it.
       vector<ustring> affected_projects;
-      ustring current_project = configuration->project;
-      Projects projects (directories_get_projects ());
-      for (unsigned int i = 0; i < projects.projects.size(); i++) {
-        configuration->project = projects.projects[i];
-        Project project (configuration);
-        if (project.get_stylesheet() == currentname) {
-          project.set_stylesheet (dialog.entered_value);
-          affected_projects.push_back (projects.projects[i]);
+      ustring current_project = genconfig.project();
+      vector <ustring> projects = projects_get_all ();
+      for (unsigned int i = 0; i < projects.size(); i++) {
+        genconfig.project_set (projects[i]);
+        ProjectConfiguration projectconfig ("");
+        if (projectconfig.stylesheet() == currentname) {
+          projectconfig.stylesheet_set (dialog.entered_value);
+          affected_projects.push_back (projects[i]);
         }
       }
-      configuration->project = current_project;
+      genconfig.project_set (current_project);
       // Rename the stylesheet itself.
       // Note: This should be done after checking on which projects have it, 
       // due to a feature in the project object that it only gives an existing
@@ -562,7 +606,7 @@ void styletree_rename_stylesheet (Configuration * configuration, GtkWidget * par
       stylesheet_copy (currentname, dialog.entered_value);
       stylesheet_delete (currentname);
       // Store new name in configuration.
-      configuration->stylesheet = dialog.entered_value;
+      genconfig.stylesheet_set (dialog.entered_value);
       // If projects were affected, mention that.
       if (affected_projects.size() > 0) {
         ustring message = "The stylesheets attached to the following projects were changed along:\n\n";
@@ -578,7 +622,7 @@ void styletree_rename_stylesheet (Configuration * configuration, GtkWidget * par
 }
 
 
-bool styletree_import_stylesheet (Configuration * configuration, GtkWidget * parent)
+bool styletree_import_stylesheet (GtkWidget * parent)
 {
   bool returnvalue = false;
   GtkWidget *dialog;
@@ -587,14 +631,15 @@ bool styletree_import_stylesheet (Configuration * configuration, GtkWidget * par
     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
   int result = gtk_dialog_run (GTK_DIALOG (dialog));
   if (result == GTK_RESPONSE_ACCEPT) {
+    GeneralConfiguration genconfig (0);
     ustring filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
     ustring name = stylesheet_import (filename);
     if (!name.empty()) {
       // Save the name of the stylesheet in two locations.
-      configuration->stylesheet = name;
-      if (!configuration->project.empty()) {
-        Project project (configuration);
-        project.set_stylesheet (name);
+      genconfig.stylesheet_set (name);
+      if (!genconfig.project().empty()) {
+        ProjectConfiguration projectconfig ("");
+        projectconfig.stylesheet_set (name);
       }
       // Open the sheet.
       returnvalue = true;
@@ -605,9 +650,10 @@ bool styletree_import_stylesheet (Configuration * configuration, GtkWidget * par
 }
 
 
-void styletree_export_stylesheet (Configuration * configuration, GtkWidget * parent)
+void styletree_export_stylesheet (GtkWidget * parent)
 {
-  ustring name = configuration->stylesheet;
+  GeneralConfiguration genconfig (0);
+  ustring name = genconfig.stylesheet();
   GtkWidget *dialog;
   dialog = gtk_file_chooser_dialog_new ("Save to a file", GTK_WINDOW (parent),
     GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -616,20 +662,22 @@ void styletree_export_stylesheet (Configuration * configuration, GtkWidget * par
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
     ustring filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
     ustring command;
-    command = "bibledit-stylesheet-export";
-    command.append (shell_quote_space (name));
-    command.append (shell_quote_space (filename));
+    command = "bibledit --export-stylesheet ";
+    command.append ("--stylesheet" + shell_quote_space (name));
+    command.append ("--file" + shell_quote_space (filename));
     system (command.c_str());
   }
   gtk_widget_destroy (dialog);  
 }
 
 
-void styletree_delete_categories (Configuration * configuration, vector<ustring> categories)
+void styletree_delete_categories (vector<ustring> categories)
 {
+  // Configuration
+  GeneralConfiguration genconfig (0);
   // Get a list of all the markers in our stylesheet.
   vector<ustring> markers;
-  markers = stylesheet_get_markers (configuration->stylesheet, NULL);
+  markers = stylesheet_get_markers (genconfig.stylesheet(), NULL);
   // Categorize them all.
   vector<UsfmCategory> usfmcategories;
   usfm_categorize_markers (markers, usfmcategories);
@@ -642,12 +690,12 @@ void styletree_delete_categories (Configuration * configuration, vector<ustring>
   for (unsigned int i = 0; i < categories.size(); i++) {
     if (categories[i] == RECENTLY_USED) {
       // If it's the recently used category, clear it.
-      styletree_clear_recently_used_styles (configuration);
+      styletree_clear_recently_used_styles ();
     } else {
       // If it's a regular category, clear it completely.
       for (unsigned int i2 = 0; i2 < markers.size(); i2++) {
         if (categories[i] == ucategories[i2]) {
-          stylesheet_delete_style (configuration->stylesheet, markers[i2]);          
+          stylesheet_delete_style (genconfig.stylesheet(), markers[i2]);          
         }
       }
     }
