@@ -24,8 +24,16 @@
 
 typedef vector <ustring> VectorUstring;
 
-
 enum NoteType {indtFootnote, indtEndnote, indtCrossreference};
 
+enum SearchResultsType {sstLoad, sstAdd, sstSubtract, sstShare};
+
+enum SearchBibleTimeType {sbttDefaultBible, sbttOpenModules, sbttBible, sbttCommentary};
+
+enum CheckSortType {cstSort0, cstSort1, cstSort2, cstSort3};
+
+enum AreaType {atRaw, atAll, atSelection};
+
+enum ImportType {itUsfm, itBibleWorks, itMechonMamre};
 
 #endif
