@@ -37,7 +37,7 @@ ustring usfm_extract (ustring & line)
       // Sometimes markers are followed by e.g. \x immediately, without a space
       // in between. This \x has been transformed to fo:inline code already, so
       // check for the "<" character too.
-      unsigned int endposition = s.find_first_of (" <\\", 1);
+      size_t endposition = s.find_first_of (" <\\", 1);
       if (endposition == string::npos) {
         line.clear();
         returnvalue = s;
@@ -127,14 +127,99 @@ opening_marker = true;
 }
 
 
-ustring get_id_from_file (const ustring & filename)
+vector<ustring> usfm_get_all_markers (const ustring& line)
+// Returns all markers that are found in the line.
 {
-  ReadText rt (filename, true);
-  unsigned int max = 10;
-  if (max >= rt.lines.size ())
-    max = rt.lines.size ();
-  for (unsigned int i = 0; i < max; i++) {
-    ustring line = trim (rt.lines[i]);
+  vector<ustring> markers;
+  ustring marker;
+  ustring line2 (line);
+  marker = usfm_extract_marker_within_line (line2);
+  while (!marker.empty()) {
+    markers.push_back (marker);
+    marker = usfm_extract_marker_within_line (line2);
+  }
+  return markers;
+}
+
+
+ustring usfm_extract_marker_within_line (ustring & line)
+/*
+Returns the usfm marker from the line, but without the first backslash, e.g. id.
+The backslash and the usfm marker are removed from the line.
+It searches within the line too.
+*/
+{
+  ustring returnvalue = trim(usfm_extract_within_line (line));
+  if (returnvalue.length () > 0)
+    returnvalue.erase (0, 1);   // Remove backslash.
+  return returnvalue;
+}
+
+
+ustring usfm_extract_within_line (ustring & line)
+/*
+This returns the usfm from the line, e.g. \id.
+The usfm is removed from the line.
+*/
+{
+  line = trim (line);
+  ustring returnvalue;
+  size_t offposition;
+  offposition = line.find ("\\");
+  if (offposition != string::npos) {
+    line.erase (0, offposition);
+    size_t endposition;
+    endposition = line.find_first_of (" *", 1);
+    if (endposition != string::npos) {
+      returnvalue = line.substr (0, ++endposition);
+      line.erase (0, endposition);
+    } else {
+      returnvalue = line;
+      line.clear();
+    }
+  }
+  return returnvalue;
+}
+
+
+void split_line_on_marker (ustring marker, ustring& line)
+// Inserts newlines just before amy place where "marker" occurs.
+{
+  marker.insert (0, "\\");
+  marker.append (" ");
+  size_t position = 0;
+  position = line.find (marker, position);
+  while (position != string::npos) {
+    line.insert (position, "\n");
+    position++;
+    position++;
+    position = line.find (marker, position);
+  }
+}
+
+
+ustring get_usfm_id_from_file (const ustring & filename)
+// Gets the Paratext ID from a file.
+// Deals with Windows "feature" to put \xEF\xBB\xBF at the start of a textfile.
+{
+  // If the file is too short, probably no USfM file.
+  if (file_get_size (filename) < 50) 
+    return "";
+  // Get contents.
+  gchar *contents;
+  g_file_get_contents (filename.c_str(), &contents, NULL, NULL);
+  // If we got nothing, then there is no ID.
+  if (!contents)
+    return "";
+  // Clear contents up.
+  contents = de_windows_notepad (contents);
+  gchar *contents_shortened = g_strndup (contents, 100);
+  ParseLine parseline (contents_shortened);
+  g_free (contents);
+  g_free (contents_shortened);
+  // Look for an ID.
+  for (unsigned int i = 0; i < parseline.lines.size(); i++) {
+    ustring line = trim (parseline.lines[i]);
     ustring marker = usfm_extract_marker (line);
     if (usfm_is_id (marker)) {
       try 
@@ -254,8 +339,7 @@ char *USFM_MARKERS_IntroductionTitlesHeadings [NUMBER_OF_MARKERS_IntroductionTit
 #define NUMBER_OF_MARKERS_IntroductionParagraphsPoetry 14
 char *USFM_MARKERS_IntroductionParagraphsPoetry [NUMBER_OF_MARKERS_IntroductionParagraphsPoetry] =
 { 
-  "ip", "ipi", "im", "imi", "imq", "ipr", "ipq", "ib", "iex", 
-  "iq", "iq1", "iq2", "iq3", "iq4"
+  "ip", "ipi", "im", "imi", "imq", "ipr", "ipq", "ib", "iex", "iq", "iq1", "iq2", "iq3", "iq4"
 };
 
 
@@ -266,12 +350,17 @@ char *USFM_MARKERS_IntroductionOtherElements [NUMBER_OF_MARKERS_IntroductionOthe
 };
 
 
-#define NUMBER_OF_MARKERS_TitlesHeadings 25
-char *USFM_MARKERS_TitlesHeadings [NUMBER_OF_MARKERS_TitlesHeadings] =
+#define NUMBER_OF_MARKERS_Titles 11
+char *USFM_MARKERS_Titles [NUMBER_OF_MARKERS_Titles] =
 { 
-  "mt", "mt1", "mt2", "mt3", "mt4", "mte", "mte1", "mte2", "mte3", "mte4", 
-  "ms", "ms1", "ms2", "ms3", "ms4",   "mr", "s", "s1", "s2", "s3", "s4", "sr", 
-  "r", "d", "sp"
+  "mt", "mt1", "mt2", "mt3", "mt4", "mte", "mte1", "mte2", "mte3", "mte4", "d"
+};
+
+
+#define NUMBER_OF_MARKERS_Headings 14
+char *USFM_MARKERS_Headings [NUMBER_OF_MARKERS_Headings] =
+{ 
+  "ms", "ms1", "ms2", "ms3", "ms4", "mr", "s", "s1", "s2", "s3", "s4", "sr", "r", "sp"
 };
 
 
@@ -282,10 +371,17 @@ char *USFM_MARKERS_ChaptersAndVerses [NUMBER_OF_MARKERS_ChaptersAndVerses] =
 };
 
 
-#define NUMBER_OF_MARKERS_ParagraphElements 27
-char *USFM_MARKERS_ParagraphElements [NUMBER_OF_MARKERS_ParagraphElements] =
+#define NUMBER_OF_MARKERS_Paragraphs 22
+char *USFM_MARKERS_Paragraphs [NUMBER_OF_MARKERS_Paragraphs] =
 { 
-  "p", "m", "pmo", "pm", "pmc", "pmr", "pi", "pi1", "pi2", "pi3", "pi4", "mi", "nb", "cls", "li", "li1", "li2", "li3", "li4", "b", "pc", "pr", "ph", "ph1", "ph2", "ph3", "ph4"
+  "p", "m", "pmo", "pm", "pmc", "pmr", "pi", "pi1", "pi2", "pi3", "pi4", "mi", "nb", "cls", "b", "pc", "pr", "ph", "ph1", "ph2", "ph3", "ph4"
+};
+
+
+#define NUMBER_OF_MARKERS_Lists 5
+char *USFM_MARKERS_Lists [NUMBER_OF_MARKERS_Lists] =
+{ 
+  "li", "li1", "li2", "li3", "li4"
 };
 
 
@@ -324,12 +420,31 @@ char *USFM_MARKERS_ExtendedStudyNotes [NUMBER_OF_MARKERS_ExtendedStudyNotes] =
 };
 
 
-#define NUMBER_OF_MARKERS_SpecialTextAndCharacterStyles 27
-char *USFM_MARKERS_SpecialTextAndCharacterStyles [NUMBER_OF_MARKERS_SpecialTextAndCharacterStyles] =
+#define NUMBER_OF_MARKERS_SpecialText 13
+char *USFM_MARKERS_SpecialText [NUMBER_OF_MARKERS_SpecialText] =
 { 
-  "qt", "nd", "tl", "dc", "bk", "sig", "pn", "wj", "k", "sis", "ord", "add", 
-  "lit", "no", "bd", "it", "bdit", "em", "sc", "pb", "fig", "pro", "w", "wh",
-  "wg", "ndx", "sls"
+  "qt", "nd", "tl", "dc", "bk", "sig", "pn", "wj", "k", "sls", "ord", "add", "lit"
+};
+
+
+#define NUMBER_OF_MARKERS_CharacterStyles 6
+char *USFM_MARKERS_CharacterStyles [NUMBER_OF_MARKERS_CharacterStyles] =
+{ 
+  "no", "bd", "it", "bdit", "em", "sc"
+};
+
+
+#define NUMBER_OF_MARKERS_SpacingsAndBreaks 1
+char *USFM_MARKERS_SpacingsAndBreaks [NUMBER_OF_MARKERS_SpacingsAndBreaks] =
+{ 
+  "pb"
+};
+
+
+#define NUMBER_OF_MARKERS_SpecialFeatures 6
+char *USFM_MARKERS_SpecialFeatures [NUMBER_OF_MARKERS_SpecialFeatures] =
+{ 
+  "fig", "pro", "w", "wh", "wg", "ndx"
 };
 
 
@@ -385,15 +500,20 @@ void usfm_categorize_markers (vector<ustring>& markers, vector<UsfmCategory>& ca
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_IntroductionTitlesHeadings, set_of_markers, USFM_MARKERS_IntroductionTitlesHeadings, ucIntroductionTitlesHeadings, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_IntroductionParagraphsPoetry, set_of_markers, USFM_MARKERS_IntroductionParagraphsPoetry, ucIntroductionParagraphsPoetry, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_IntroductionOtherElements, set_of_markers, USFM_MARKERS_IntroductionOtherElements, ucIntroductionOtherElements, markers, categories);
-  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_TitlesHeadings, set_of_markers, USFM_MARKERS_TitlesHeadings, ucTitlesHeadings, markers, categories);
+  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_Titles, set_of_markers, USFM_MARKERS_Titles, ucTitles, markers, categories);
+  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_Headings, set_of_markers, USFM_MARKERS_Headings, ucHeadings, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_ChaptersAndVerses, set_of_markers, USFM_MARKERS_ChaptersAndVerses, ucChaptersAndVerses, markers, categories);
-  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_ParagraphElements, set_of_markers, USFM_MARKERS_ParagraphElements, ucParagraphElements, markers, categories);
+  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_Paragraphs, set_of_markers, USFM_MARKERS_Paragraphs, ucParagraphs, markers, categories);
+  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_Lists, set_of_markers, USFM_MARKERS_Lists, ucLists, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_PoetryElements, set_of_markers, USFM_MARKERS_PoetryElements, ucPoetryElements, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_TableElements, set_of_markers, USFM_MARKERS_TableElements, ucTableElements, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_Footnotes, set_of_markers, USFM_MARKERS_Footnotes, ucFootnotes, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_CrossReferences, set_of_markers, USFM_MARKERS_CrossReferences, ucCrossReferences, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_ExtendedStudyNotes, set_of_markers, USFM_MARKERS_ExtendedStudyNotes, ucExtendedStudyNotes, markers, categories);
-  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_SpecialTextAndCharacterStyles, set_of_markers, USFM_MARKERS_SpecialTextAndCharacterStyles, ucSpecialTextAndCharacterStyles, markers, categories);
+  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_SpecialText, set_of_markers, USFM_MARKERS_SpecialText, ucSpecialText, markers, categories);
+  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_CharacterStyles, set_of_markers, USFM_MARKERS_CharacterStyles, ucCharacterStyles, markers, categories);
+  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_SpacingsAndBreaks, set_of_markers, USFM_MARKERS_SpacingsAndBreaks, ucSpacingsAndBreaks, markers, categories);
+  usfm_categorize_markers_internal (NUMBER_OF_MARKERS_SpecialFeatures, set_of_markers, USFM_MARKERS_SpecialFeatures, ucSpecialFeatures, markers, categories);
   usfm_categorize_markers_internal (NUMBER_OF_MARKERS_PeripheralMaterials, set_of_markers, USFM_MARKERS_PeripheralMaterials, ucPeripheralMaterials, markers, categories);
   // Dump any remaining markers in the "Other" category.
   vector<ustring> other_markers (set_of_markers.begin(), set_of_markers.end());
@@ -428,9 +548,14 @@ ustring usfm_get_category_name (UsfmCategory category)
       name = "Introduction Other Elements";
       break;
     } 
-    case ucTitlesHeadings :
+    case ucTitles :
     {
-      name = "Titles and Headings";
+      name = "Titles";
+      break;
+    }
+    case ucHeadings :
+    {
+      name = "Headings";
       break;
     }
     case ucChaptersAndVerses :
@@ -438,9 +563,14 @@ ustring usfm_get_category_name (UsfmCategory category)
       name = "Chapters and Verses";
       break;
     } 
-    case ucParagraphElements :
+    case ucParagraphs :
     {
-      name = "Paragraph Elements";
+      name = "Paragraphs";
+      break;
+    } 
+    case ucLists :
+    {
+      name = "Lists";
       break;
     } 
     case ucPoetryElements :
@@ -468,9 +598,24 @@ ustring usfm_get_category_name (UsfmCategory category)
       name = "Extended Study Notes";
       break;
     } 
-    case ucSpecialTextAndCharacterStyles :
+    case ucSpecialText :
     {
-      name = "Special Text and Character Styles";
+      name = "Special Text";
+      break;
+    }
+    case ucCharacterStyles :
+    {
+      name = "Character Styles";
+      break;
+    }
+    case ucSpacingsAndBreaks :
+    {
+      name = "Spacings and Breaks";
+      break;
+    }
+    case ucSpecialFeatures :
+    {
+      name = "Special Features";
       break;
     }
     case ucPeripheralMaterials :
