@@ -28,6 +28,8 @@
 #include "shell.h"
 #include "unixwrappers.h"
 #include "uname.h"
+#include "books.h"
+#include "generalconfig.h"
 
 
 #define STAGE_ZERO 0
@@ -62,10 +64,9 @@ WindowsOutpost::~WindowsOutpost ()
 }
 
 
-void WindowsOutpost::Start (Configuration * configuration)
+void WindowsOutpost::Start ()
 // This effectually starts the whole system.
 {
-  myconfiguration = configuration;
   thread_run = true;
   g_thread_create (GThreadFunc (thread_start), gpointer (this), false, NULL);
 }
@@ -87,6 +88,7 @@ void WindowsOutpost::thread_start (gpointer data)
 
 void WindowsOutpost::thread_main ()
 {
+  GeneralConfiguration genconfig (0);
   thread_running = true;
   while (thread_run) {
     switch (stage)
@@ -143,7 +145,7 @@ void WindowsOutpost::thread_main ()
         // If BibleWorks does not run, and Outpost tries to contact it, it 
         // crashes in Wine and becomes unusable. Therefore always check whether
         // BibleWorks runs.
-        if (program_is_running (myconfiguration->bibleworks_executable))
+        if (program_is_running (genconfig.bibleworks_executable()))
           send_scheduled (bibleworks_reference_set_value);
         send_scheduled (santafefocus_reference_set_value);
         send_scheduled (santafefocus_word_set_value);
@@ -315,21 +317,27 @@ immediately with a null string.
 void WindowsOutpost::BibleWorksReferenceSet (const ustring& book, const ustring& chapter, const ustring& verse)
 // Schedules a reference to be sent to BibleWorks.
 {
+  unsigned int index = books_english_to_id (book);
+  if (!index) 
+    return;
   // As it does not respond properly to chapter/verses that are 0, care for that.
-  unsigned int index = english_id_to_index (book);
   ustring goto_ch = chapter;
   if (goto_ch == "0") goto_ch = "1";
   ustring goto_vs = verse;
   if (goto_vs == "0") goto_vs = "1";
-  bibleworks_reference_set_value = "BibleWorksReferenceSet " + index_to_bibleworks_id (index) + " " + goto_ch + ":" + goto_vs;
+  ustring bw_book = books_id_to_bibleworks (index);
+  if (!bw_book.empty())
+    bibleworks_reference_set_value = "BibleWorksReferenceSet " + bw_book + " " + goto_ch + ":" + goto_vs;
 }
 
 
 void WindowsOutpost::SantaFeFocusReferenceSet (const ustring& book, const ustring& chapter, const ustring& verse)
 // Schedules a reference to be sent to the santa fe focus system.
 {
-  unsigned int index = english_id_to_index (book);
-  santafefocus_reference_set_value = "SantaFeFocusReferenceSet " + index_to_paratext_id (index) + " " + chapter + ":" + verse;
+  unsigned int index = books_english_to_id (book);
+  if (index) {
+    santafefocus_reference_set_value = "SantaFeFocusReferenceSet " + books_id_to_paratext (index) + " " + chapter + ":" + verse;
+  }
 }
 
 
@@ -894,3 +902,13 @@ Function IsW2KorNewer() As Boolean
     IsW2KorNewer = OS.dwMajorVersion > 4
 End Function
 */
+
+
+/* Todo starting outpost cxoffice
+/opt/cxoffice/bin/wine --bottle win98 --cx-app bwoutpost.exe
+
+Differences between regular wine and cx wine can be found by running
+wine --help
+The output is different in both cases.
+
+*/
