@@ -25,15 +25,14 @@
 #include "libraries.h"
 #include <glib.h>
 #include <gtk/gtk.h>
-#include "configuration.h"
 #include "notecaller.h"
-#include "progress.h"
+#include "progresswindow.h"
 
 
-bool xep_present (GtkWidget * window, Configuration * configuration);
-ustring xep_convert_from_fo_to_xep (Configuration * configuration, const ustring& xslfo, const ustring& xep, const ustring& logfile);
-void xep_convert_footnote_callers (const ustring& filename, NoteCaller * footnotecaller, NoteCaller * crossreferencecaller, Progress * progress);
-ustring xep_convert_from_xep_to_pdf (Configuration * configuration, const ustring& xep, const ustring& pdf, const ustring& logfile);
+bool xep_present (GtkWidget * window);
+ustring xep_convert_from_fo_to_xep (const ustring& xslfo, const ustring& xep, const ustring& logfile);
+void xep_convert_footnote_callers (const ustring& filename, NoteCaller * footnotecaller, NoteCaller * crossreferencecaller, ProgressWindow& progresswindow);
+ustring xep_convert_from_xep_to_pdf (const ustring& xep, const ustring& pdf, const ustring& logfile);
 
 
 #endif
