@@ -25,7 +25,6 @@
 #include "libraries.h"
 #include "usfm.h"
 #include "xmlfo-utils.h"
-#include "configuration.h"
 #include "notecaller.h"
 #include "xslfofootnoteparagraph.h"
 
@@ -33,11 +32,10 @@
 class XslFoEndnote
 {
 public:
-  XslFoEndnote (const Usfm& usfm, Configuration * configuration, bool show);
+  XslFoEndnote (const Usfm& usfm, bool show);
   ~XslFoEndnote ();
   void transform (XmlFoBlock * xmlfoblock, ustring& line);
 private:
-  Configuration * myconfiguration;
   bool myshow;
   ustring opening_marker;
   ustring closing_marker;
